@@ -1,11 +1,11 @@
 /*
- * analysis.c
+ * Copyright (C) 2000-2004 Joern Thyssen <joern@thyssen.nu>
+ * Copyright (C) 2000-2023 the AUTHORS
  *
- * by Joern Thyssen <joern@thyssen.nu>, 2000
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 3 or later of the GNU General Public License as
- * published by the Free Software Foundation.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,12 +13,36 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
- * $Id: analysis.c,v 1.240 2014/07/27 01:28:30 mdpetch Exp $
+ * $Id: analysis.c,v 1.263 2023/09/05 20:34:21 plm Exp $
  */
 
+/*
+02/2023: Isaac Keslassy: introduced the "background analysis"
+feature, together with a new button in the main toolbar: "Analyse"
+- button: "Analyse":
+    - This is to analyse the current match.
+    - Define options for what it does (in Settings>Analysis):
+        - "background analysis" checkbox:
+            * unchecked => regular "analyse match" (blocking, like today), vs.
+            * checked => we can browse at the same time and check the analysis
+        - "automatic add-to-database" checkbox:
+            * Unchecked = like today, vs
+            * Checked = automatic add-to-db at end of analysis (and we remove the add-to-db
+                button from the statistics page)
+- 2nd button: "Analyze file": 
+    - This is for a file, e.g. a match that we just played online. 
+    - Can launch either one of 3 options (defined in Settings>Analysis):
+        1) Regular "batch analysis": blocking, like today (+add-to-db)
+        2) "Single-file analysis": analysis of 1 chosen file, can be in 
+            background and/or add-to-db based on defined settings
+        3) "Smart analysis": pick the newest file in the preferred folder that gnubg 
+            recognizes, then can be in background and/or add-to-db based on defined settings
+When analyzing in the background, various menus are disabled so the user does not launch
+    another analysis in the middle.
+*/
+
 #include "config.h"
 
 #include <glib.h>
@@ -28,15 +52,13 @@
 #include "backgammon.h"
 #include "drawboard.h"
 #include "eval.h"
-#if USE_GTK
+#if defined(USE_GTK)
 #include "gtkgame.h"
 #endif
 #include "positionid.h"
-#include "rollout.h"
 #include "analysis.h"
 #include "sound.h"
 #include "matchequity.h"
-#include "export.h"
 #include "formatgs.h"
 #include "progress.h"
 #include "multithread.h"
@@ -87,7 +109,7 @@ GetRating(const float rError)
 }
 
 static float
-LuckFirst(const TanBoard anBoard, const int n0, const int n1, const cubeinfo * pci, const evalcontext * pec)
+LuckFirst(const TanBoard anBoard, const int n0, const int n1, cubeinfo * pci, const evalcontext * pec)
 {
 
     TanBoard anBoardTemp;
@@ -107,15 +129,17 @@ LuckFirst(const TanBoard anBoard, const int n0, const int n1, const cubeinfo * p
 
             /* Find the best move for each roll at ply 0 only. */
             if (FindnSaveBestMoves(&ml, i + 1, j + 1, (ConstTanBoard) anBoardTemp, NULL, 0.0f,
-                                   (cubeinfo *) pci, (evalcontext *) pec, defaultFilters) < 0)
-                return (float) ERR_VAL;
+                                   pci, pec, defaultFilters) < 0) {
+                g_free(ml.amMoves);
+                return ERR_VAL;
+            }
 
             if (!ml.cMoves) {
 
                 SwapSides(anBoardTemp);
 
-                if (GeneralEvaluationE(ar, (ConstTanBoard) anBoardTemp, &ciOpp, (evalcontext *) pec) < 0)
-                    return (float) ERR_VAL;
+                if (GeneralEvaluationE(ar, (ConstTanBoard) anBoardTemp, &ciOpp, pec) < 0)
+                    return ERR_VAL;
 
                 if (pec->fCubeful) {
                     if (pci->nMatchTo)
@@ -127,7 +151,7 @@ LuckFirst(const TanBoard anBoard, const int n0, const int n1, const cubeinfo * p
 
             } else {
                 aar[i][j] = ml.amMoves[0].rScore;
-                free(ml.amMoves);
+                g_free(ml.amMoves);
             }
 
             rMean += aar[i][j];
@@ -143,15 +167,17 @@ LuckFirst(const TanBoard anBoard, const int n0, const int n1, const cubeinfo * p
 
             /* Find the best move for each roll at ply 0 only. */
             if (FindnSaveBestMoves(&ml, i + 1, j + 1, (ConstTanBoard) anBoardTemp, NULL, 0.0f,
-                                   &ciOpp, (evalcontext *) pec, defaultFilters) < 0)
-                return (float) ERR_VAL;
+                                   &ciOpp, pec, defaultFilters) < 0) {
+                g_free(ml.amMoves);
+                return ERR_VAL;
+            }
 
             if (!ml.cMoves) {
 
                 SwapSides(anBoardTemp);
 
-                if (GeneralEvaluationE(ar, (ConstTanBoard) anBoardTemp, (cubeinfo *) pci, (evalcontext *) pec) < 0)
-                    return (float) ERR_VAL;
+                if (GeneralEvaluationE(ar, (ConstTanBoard) anBoardTemp, pci, pec) < 0)
+                    return ERR_VAL;
 
                 if (pec->fCubeful) {
                     if (pci->nMatchTo)
@@ -163,7 +189,7 @@ LuckFirst(const TanBoard anBoard, const int n0, const int n1, const cubeinfo * p
 
             } else {
                 aar[i][j] = -ml.amMoves[0].rScore;
-                free(ml.amMoves);
+                g_free(ml.amMoves);
             }
 
             rMean += aar[i][j];
@@ -196,15 +222,17 @@ LuckNormal(const TanBoard anBoard, const int n0, const int n1, const cubeinfo *
 
             /* Find the best move for each roll at ply 0 only. */
             if (FindnSaveBestMoves(&ml, i + 1, j + 1, (ConstTanBoard) anBoardTemp, NULL, 0.0f,
-                                   (cubeinfo *) pci, (evalcontext *) pec, defaultFilters) < 0)
-                return (float) ERR_VAL;
+                                   pci, pec, defaultFilters) < 0) {
+                g_free(ml.amMoves);
+                return ERR_VAL;
+            }
 
             if (!ml.cMoves) {
 
                 SwapSides(anBoardTemp);
 
-                if (GeneralEvaluationE(ar, (ConstTanBoard) anBoardTemp, &ciOpp, (evalcontext *) pec) < 0)
-                    return (float) ERR_VAL;
+                if (GeneralEvaluationE(ar, (ConstTanBoard) anBoardTemp, &ciOpp, pec) < 0)
+                    return ERR_VAL;
 
                 if (pec->fCubeful) {
                     if (pci->nMatchTo)
@@ -216,7 +244,7 @@ LuckNormal(const TanBoard anBoard, const int n0, const int n1, const cubeinfo *
 
             } else {
                 aar[i][j] = ml.amMoves[0].rScore;
-                free(ml.amMoves);
+                g_free(ml.amMoves);
             }
 
             rMean += (i == j) ? aar[i][j] : aar[i][j] * 2.0f;
@@ -238,10 +266,10 @@ LuckAnalysis(const TanBoard anBoard, int n0, int n1, matchstate * pms)
     InitBoard(init_board, pms->bgv);
     is_init_board = !memcmp(init_board, pms->anBoard, 2 * 25 * sizeof(int));
 
-    if (n0-- < n1--)
+    if (n0-- < n1--)            /* -- because as input n0 and n1 are dice [1..6] but in calls to LuckXX() they are array indexes [0..5] */
         swap(&n0, &n1);
 
-    if (is_init_board && n0 != n1)
+    if (is_init_board && n0 != n1)      /* FIXME: this fails if we return to the initial position after a few moves */
         return LuckFirst(anBoard, n0, n1, &ci, &ecLuck);
     else
         return LuckNormal(anBoard, n0, n1, &ci, &ecLuck);
@@ -294,7 +322,7 @@ updateStatcontext(statcontext * psc, const moverecord * pmr, const matchstate *
 {
     cubeinfo ci;
     static positionkey key;
-    float rSkill, rChequerSkill, rCost;
+    float rSkill, rCost;
     unsigned int i;
     float arDouble[4];
     const xmovegameinfo *pmgi = &((moverecord *) plGame->plNext->p)->g;
@@ -357,7 +385,7 @@ updateStatcontext(statcontext * psc, const moverecord * pmr, const matchstate *
 
                 rSkill = arDouble[OUTPUT_NODOUBLE] - arDouble[OUTPUT_OPTIMAL];
 
-                rCost = pms->nMatchTo ? eq2mwc(rSkill, &ci) - eq2mwc(0.0f, &ci) : pms->nCube * rSkill;
+                rCost = pms->nMatchTo ? eq2mwc(rSkill, &ci) - eq2mwc(0.0f, &ci) : (float) pms->nCube * rSkill;
 
                 if (arDouble[OUTPUT_TAKE] > 1.0f) {
                     /* missed double above cash point */
@@ -381,9 +409,9 @@ updateStatcontext(statcontext * psc, const moverecord * pmr, const matchstate *
         /*
          * update luck statistics for roll
          */
-        if (fAnalyseDice && pmr->rLuck != (float) ERR_VAL) {
+        if (fAnalyseDice && pmr->rLuck != ERR_VAL) {
 
-            float r = pms->nMatchTo ? eq2mwc(pmr->rLuck, &ci) - eq2mwc(0.0f, &ci) : pms->nCube * pmr->rLuck;
+            float r = pms->nMatchTo ? eq2mwc(pmr->rLuck, &ci) - eq2mwc(0.0f, &ci) : (float) pms->nCube * pmr->rLuck;
 
             psc->arLuck[pmr->fPlayer][0] += pmr->rLuck;
             psc->arLuck[pmr->fPlayer][1] += r;
@@ -412,10 +440,12 @@ updateStatcontext(statcontext * psc, const moverecord * pmr, const matchstate *
         if (fAnalyseMove && (pmr->esChequer.et != EVAL_NONE || pmr->n.anMove[0] < 0)) {
             /* find skill */
             TanBoard anBoardMove;
+            float rChequerSkill = 0.0f;
+
             memcpy(anBoardMove, pms->anBoard, sizeof(anBoardMove));
             ApplyMove(anBoardMove, pmr->n.anMove, FALSE);
             PositionKey((ConstTanBoard) anBoardMove, &key);
-            rChequerSkill = 0.0f;
+
             if (pmr->ml.amMoves) {
                 for (i = 0; i < pmr->ml.cMoves; i++)
 
@@ -428,7 +458,7 @@ updateStatcontext(statcontext * psc, const moverecord * pmr, const matchstate *
             }
             /* update statistics */
 
-            rCost = pms->nMatchTo ? eq2mwc(rChequerSkill, &ci) - eq2mwc(0.0f, &ci) : pms->nCube * rChequerSkill;
+            rCost = pms->nMatchTo ? eq2mwc(rChequerSkill, &ci) - eq2mwc(0.0f, &ci) : (float) pms->nCube * rChequerSkill;
 
             psc->anMoves[pmr->fPlayer][Skill(rChequerSkill)]++;
 
@@ -467,7 +497,7 @@ updateStatcontext(statcontext * psc, const moverecord * pmr, const matchstate *
             if (rSkill < 0.0f) {
                 /* it was not a double */
 
-                rCost = pms->nMatchTo ? eq2mwc(rSkill, &ci) - eq2mwc(0.0f, &ci) : pms->nCube * rSkill;
+                rCost = pms->nMatchTo ? eq2mwc(rSkill, &ci) - eq2mwc(0.0f, &ci) : (float) pms->nCube * rSkill;
 
                 if (arDouble[OUTPUT_NODOUBLE] > 1.0f) {
                     /* wrong double above too good point */
@@ -507,7 +537,7 @@ updateStatcontext(statcontext * psc, const moverecord * pmr, const matchstate *
 
                 rSkill = -arDouble[OUTPUT_TAKE] - -arDouble[OUTPUT_DROP];
 
-                rCost = pms->nMatchTo ? eq2mwc(rSkill, &ci) - eq2mwc(0.0f, &ci) : pms->nCube * rSkill;
+                rCost = pms->nMatchTo ? eq2mwc(rSkill, &ci) - eq2mwc(0.0f, &ci) : (float) pms->nCube * rSkill;
 
                 psc->anCubeWrongTake[pmr->fPlayer]++;
                 psc->arErrorWrongTake[pmr->fPlayer][0] -= rSkill;
@@ -534,7 +564,7 @@ updateStatcontext(statcontext * psc, const moverecord * pmr, const matchstate *
 
                 rSkill = -arDouble[OUTPUT_DROP] - -arDouble[OUTPUT_TAKE];
 
-                rCost = pms->nMatchTo ? eq2mwc(rSkill, &ci) - eq2mwc(0.0f, &ci) : pms->nCube * rSkill;
+                rCost = pms->nMatchTo ? eq2mwc(rSkill, &ci) - eq2mwc(0.0f, &ci) : (float) pms->nCube * rSkill;
 
                 psc->anCubeWrongPass[pmr->fPlayer]++;
                 psc->arErrorWrongPass[pmr->fPlayer][0] -= rSkill;
@@ -552,9 +582,9 @@ updateStatcontext(statcontext * psc, const moverecord * pmr, const matchstate *
 
 
         GetMatchStateCubeInfo(&ci, pms);
-        if (fAnalyseDice && pmr->rLuck != (float) ERR_VAL) {
+        if (fAnalyseDice && pmr->rLuck != ERR_VAL) {
 
-            float r = pms->nMatchTo ? eq2mwc(pmr->rLuck, &ci) - eq2mwc(0.0f, &ci) : pms->nCube * pmr->rLuck;
+            float r = pms->nMatchTo ? eq2mwc(pmr->rLuck, &ci) - eq2mwc(0.0f, &ci) : (float) pms->nCube * pmr->rLuck;
 
             psc->arLuck[pmr->fPlayer][0] += pmr->rLuck;
             psc->arLuck[pmr->fPlayer][1] += r;
@@ -673,7 +703,7 @@ AnalyzeMove(moverecord * pmr, matchstate * pms, const listOLD * plParentGame,
             if (cmp_evalsetup(pesChequer, &pmr->esChequer) > 0) {
 
                 if (pmr->ml.cMoves) {
-                    free(pmr->ml.amMoves);
+                    g_free(pmr->ml.amMoves);
                     pmr->ml.amMoves = NULL;
                 }
 
@@ -685,12 +715,15 @@ AnalyzeMove(moverecord * pmr, matchstate * pms, const listOLD * plParentGame,
                     if (FindnSaveBestMoves(&ml, pmr->anDice[0],
                                            pmr->anDice[1],
                                            (ConstTanBoard) pms->anBoard, &key,
-                                           arSkillLevel[SKILL_DOUBTFUL], &ci, &pesChequer->ec, aamf) < 0)
+                                           arSkillLevel[SKILL_DOUBTFUL], &ci, &pesChequer->ec, aamf) < 0) {
+                        g_free(ml.amMoves);
                         return -1;
+                    }
                     MT_Exclusive();
                     CopyMoveList(&pmr->ml, &ml);
-                    if (ml.cMoves)
-                        free(ml.amMoves);
+                    if (ml.cMoves) {
+                        g_free(ml.amMoves);
+                    }
                 }
 
             }
@@ -718,7 +751,7 @@ AnalyzeMove(moverecord * pmr, matchstate * pms, const listOLD * plParentGame,
 
         dt = DoubleType(pms->fDoubled, pms->fMove, pms->fTurn);
 
-        if (dt != DT_NORMAL)
+        if (dt > DT_NORMAL)     /* TODO: analyse beavers */
             break;
 
         /* cube action */
@@ -754,7 +787,7 @@ AnalyzeMove(moverecord * pmr, matchstate * pms, const listOLD * plParentGame,
 
                 pmr->stCube = Skill(rSkill);
             } else if (doubleError)
-                *doubleError = (float) ERR_VAL;
+                *doubleError = ERR_VAL;
         }
 
         if (psc)
@@ -769,10 +802,10 @@ AnalyzeMove(moverecord * pmr, matchstate * pms, const listOLD * plParentGame,
             break;
 
         tt = (taketype) DoubleType(pms->fDoubled, pms->fMove, pms->fTurn);
-        if (tt > TT_NORMAL)
+        if (tt > TT_NORMAL)     /* TODO: analyse beavers */
             break;
 
-        if (fAnalyseCube && pmgi->fCubeUse && doubleError && (*doubleError != (float) ERR_VAL)) {
+        if (fAnalyseCube && pmgi->fCubeUse && doubleError && (*doubleError != ERR_VAL)) {
             GetMatchStateCubeInfo(&ci, pms);
             pmr->stCube = Skill(-*doubleError);
         }
@@ -789,10 +822,10 @@ AnalyzeMove(moverecord * pmr, matchstate * pms, const listOLD * plParentGame,
             break;
 
         tt = (taketype) DoubleType(pms->fDoubled, pms->fMove, pms->fTurn);
-        if (tt > TT_NORMAL)
+        if (tt > TT_NORMAL)     /* TODO: analyse beavers */
             break;
 
-        if (fAnalyseCube && pmgi->fCubeUse && doubleError && (*doubleError != (float) ERR_VAL)) {
+        if (fAnalyseCube && pmgi->fCubeUse && doubleError && (*doubleError != ERR_VAL)) {
             GetMatchStateCubeInfo(&ci, pms);
             pmr->stCube = Skill(*doubleError);
         }
@@ -933,7 +966,6 @@ AnalyseMoveMT(Task * task)
 static int
 AnalyzeGame(listOLD * plGame, int wait)
 {
-    int result;
     unsigned int i;
     listOLD *pl = plGame->plNext;
     moverecord *pmr = pl->p;
@@ -946,7 +978,7 @@ AnalyzeGame(listOLD * plGame, int wait)
     g_assert(pmr->mt == MOVE_GAMEINFO);
     if (AnalyzeMove(pmr, &msAnalyse, plGame, psc,
                     &esAnalysisChequer, &esAnalysisCube, aamfAnalysis, afAnalysePlayers, NULL) < 0)
-        return -1;              /* Interupted */
+        return -1;              /* Interrupted */
 
     numMoves--;                 /* Done one - the gameinfo */
 
@@ -1000,6 +1032,8 @@ AnalyzeGame(listOLD * plGame, int wait)
     g_assert(pl->plNext == plGame);
 
     if (wait) {
+        int result;
+
         multi_debug("wait for all task: analysis");
         result = MT_WaitForTasks(UpdateProgressBar, 250, fAutoSaveAnalysis);
 
@@ -1026,11 +1060,11 @@ UpdateVariance(float *prVariance, const float rSum, const float rSumAdd, const i
          * for formula */
 
         float rDelta = rSumAdd;
-        float rMuNew = rSum / nGames;
-        float rMuOld = (rSum - rDelta) / (nGames - 1);
+        float rMuNew = rSum / (float) nGames;
+        float rMuOld = (rSum - rDelta) / (float) (nGames - 1);
         float rDeltaMu = rMuNew - rMuOld;
 
-        *prVariance = *prVariance * (1.0f - 1.0f / (nGames - 1.0f)) + nGames * rDeltaMu * rDeltaMu;
+        *prVariance = *prVariance * (1.0f - 1.0f / ((float) (nGames - 1))) + (float) nGames * rDeltaMu * rDeltaMu;
 
         return;
 
@@ -1046,6 +1080,8 @@ AddStatcontext(const statcontext * pscA, statcontext * pscB)
 
     int i, j;
 
+    MT_Exclusive();
+
     pscB->nGames++;
 
     pscB->fMoves |= pscA->fMoves;
@@ -1106,7 +1142,7 @@ AddStatcontext(const statcontext * pscA, statcontext * pscB)
         }
 
     }
-
+    MT_Release();
 }
 
 static int
@@ -1151,13 +1187,31 @@ CommandAnalyseGame(char *UNUSED(sz))
     fStore_crawford = ms.fCrawford;
     nMoves = NumberMovesGame(plGame);
 
-    ProgressStartValue(_("Analysing game; move:"), nMoves);
+ /* see explanations in CommandAnalyseMatch()*/
+#if defined(USE_GTK)
+    if (fBackgroundAnalysis && fX) {
+        fAnalysisRunning = TRUE;
+        ProgressStartValue(_("Background analysis. Browsing-only mode: "
+          "feel free to browse and check the early analysis results."), nMoves);
+        ShowBoard(); /* hide unallowd toolbar items*/
+        GTKRegenerateGames(); /* hide unallowed menu items*/
+    } else
+#endif
+        ProgressStartValue(_("Analysing game"), nMoves);
 
     AnalyzeGame(plGame, TRUE);
 
     ProgressEnd();
 
-#if USE_GTK
+    if (fBackgroundAnalysis) {
+        fAnalysisRunning = FALSE;
+        ShowBoard(); /* hide unallowd toolbar items*/
+#if defined(USE_GTK)
+        GTKRegenerateGames(); /* hide unallowed menu items*/
+#endif
+    }
+
+#if defined(USE_GTK)
     if (fX)
         ChangeGame(NULL);
 #endif
@@ -1185,7 +1239,26 @@ CommandAnalyseMatch(char *UNUSED(sz))
     fStore_crawford = ms.fCrawford;
     nMoves = NumberMovesMatch(&lMatch);
 
-    ProgressStartValue(_("Analysing match; move:"), nMoves);
+    /* if we analyze in the background, we turn on a global flag
+       to disable all sorts of buttons during the analysis */
+
+#if defined(USE_GTK)
+    if(fBackgroundAnalysis && fX) {
+        fAnalysisRunning = TRUE;
+        ProgressStartValue(_("Background analysis. Browsing-only mode: "
+        "feel free to browse and check the early analysis results."), nMoves);
+        ShowBoard(); /* hide unallowd toolbar items*/
+        GTKRegenerateGames(); /* hide unallowed menu items*/
+    }
+    else
+#endif
+    {
+        /* this was supposed to show nMoves, but it's not used at the end;
+        on the right side we see "n/nTotal"; so we update the text
+        */
+        // ProgressStartValue(_("Analysing match; move:"), nMoves);
+        ProgressStartValue(_("Analysing match"), nMoves);
+    }
 
     IniStatcontext(&scMatch);
 
@@ -1207,7 +1280,16 @@ CommandAnalyseMatch(char *UNUSED(sz))
 
     ProgressEnd();
 
-#if USE_GTK
+#if defined(USE_GTK)
+    if (fBackgroundAnalysis && fX) {
+        fAnalysisRunning = FALSE;
+         // CalculateBoard();
+        ShowBoard(); /* show toolbar items*/
+        GTKRegenerateGames(); /* show menu items*/
+    }
+#endif
+
+#if defined(USE_GTK)
     if (fX)
         ChangeGame(NULL);
 #endif
@@ -1288,9 +1370,9 @@ IniStatcontext(statcontext * psc)
 extern float
 relativeFibsRating(float r, int n)
 {
-    float const x = -2000.0f / (float) sqrt(1.0 * n) * (float) log10(1.0 / r - 1.0);
+    float const x = -2000.0f / sqrtf((float) n) * log10f(1.0f / r - 1.0f);
 
-    return (x < -2100) ? -2100 : x;
+    return (x < -2100.0f) ? -2100.0f : x;
 }
 
 /*
@@ -1306,7 +1388,7 @@ extern float
 absoluteFibsRatingChequer(const float rChequer, const int n)
 {
 
-    return rChequer * (8798.0f + 25526.0f / (n));
+    return rChequer * (8798.0f + 25526.0f / (float) n);
 
 }
 
@@ -1324,7 +1406,7 @@ extern float
 absoluteFibsRatingCube(const float rCube, const int n)
 {
 
-    return rCube * (863.0f - 519.0f / n);
+    return rCube * (863.0f - 519.0f / (float) n);
 
 }
 
@@ -1362,7 +1444,7 @@ getMWCFromError(const statcontext * psc, float aaaar[3][2][2][2])
             aaaar[CHEQUERPLAY][TOTAL][i][j] = psc->arErrorCheckerplay[i][j];
 
             if (psc->anUnforcedMoves[i])
-                aaaar[CHEQUERPLAY][PERMOVE][i][j] = aaaar[0][0][i][j] / psc->anUnforcedMoves[i];
+                aaaar[CHEQUERPLAY][PERMOVE][i][j] = aaaar[0][0][i][j] / (float) psc->anUnforcedMoves[i];
             else
                 aaaar[CHEQUERPLAY][PERMOVE][i][j] = 0.0f;
 
@@ -1376,7 +1458,7 @@ getMWCFromError(const statcontext * psc, float aaaar[3][2][2][2])
                 + psc->arErrorWrongPass[i][j];
 
             if (psc->anCloseCube[i])
-                aaaar[CUBEDECISION][PERMOVE][i][j] = aaaar[CUBEDECISION][TOTAL][i][j] / psc->anCloseCube[i];
+                aaaar[CUBEDECISION][PERMOVE][i][j] = aaaar[CUBEDECISION][TOTAL][i][j] / (float) psc->anCloseCube[i];
             else
                 aaaar[CUBEDECISION][PERMOVE][i][j] = 0.0f;
 
@@ -1388,20 +1470,24 @@ getMWCFromError(const statcontext * psc, float aaaar[3][2][2][2])
 
             if (psc->anUnforcedMoves[i] + psc->anCloseCube[i])
                 aaaar[COMBINED][PERMOVE][i][j] =
-                    aaaar[COMBINED][TOTAL][i][j] / (psc->anUnforcedMoves[i] + psc->anCloseCube[i]);
+                    aaaar[COMBINED][TOTAL][i][j] / (float) (psc->anUnforcedMoves[i] + psc->anCloseCube[i]);
             else
                 aaaar[COMBINED][PERMOVE][i][j] = 0.0f;
 
         }
 }
 
+#define DSCFORMAT "%-40s %-23s %-23s\n"
+
 extern void
-DumpStatcontext(char *szOutput, const statcontext * psc, const char *pl, const char *op, int nMatchTo)
+DumpStatcontext(char *szOutput, const statcontext * psc, const char *player, const char *op, int nMatchTo)
 {
 
     /* header */
 
-    sprintf(szOutput, "%-40s %-23s %-23s\n\n", _("Player"), pl, op);
+    sprintf(szOutput, DSCFORMAT, _("Player"), player, op);
+
+    strcat(szOutput, "\n");
 
     if (psc->fMoves) {
         GList *list = formatGS(psc, nMatchTo, FORMATGS_CHEQUER);
@@ -1414,7 +1500,7 @@ DumpStatcontext(char *szOutput, const statcontext * psc, const char *pl, const c
 
             char **asz = pl->data;
 
-            sprintf(strchr(szOutput, 0), "%-40s %-23s %-23s\n", asz[0], asz[1], asz[2]);
+            sprintf(strchr(szOutput, 0), DSCFORMAT, asz[0], asz[1], asz[2]);
 
         }
 
@@ -1436,7 +1522,7 @@ DumpStatcontext(char *szOutput, const statcontext * psc, const char *pl, const c
 
             char **asz = pl->data;
 
-            sprintf(strchr(szOutput, 0), "%-40s %-23s %-23s\n", asz[0], asz[1], asz[2]);
+            sprintf(strchr(szOutput, 0), DSCFORMAT, asz[0], asz[1], asz[2]);
 
         }
 
@@ -1458,7 +1544,7 @@ DumpStatcontext(char *szOutput, const statcontext * psc, const char *pl, const c
 
             char **asz = pl->data;
 
-            sprintf(strchr(szOutput, 0), "%-40s %-23s %-23s\n", asz[0], asz[1], asz[2]);
+            sprintf(strchr(szOutput, 0), DSCFORMAT, asz[0], asz[1], asz[2]);
 
         }
 
@@ -1480,7 +1566,7 @@ DumpStatcontext(char *szOutput, const statcontext * psc, const char *pl, const c
 
             char **asz = pl->data;
 
-            sprintf(strchr(szOutput, 0), "%-40s %-23s %-23s\n", asz[0], asz[1], asz[2]);
+            sprintf(strchr(szOutput, 0), DSCFORMAT, asz[0], asz[1], asz[2]);
 
         }
 
@@ -1492,11 +1578,11 @@ DumpStatcontext(char *szOutput, const statcontext * psc, const char *pl, const c
 extern void
 CommandShowStatisticsMatch(char *UNUSED(sz))
 {
-    char szOutput[4096];
+    char szOutput[STATCONTEXT_MAXSIZE];
 
     updateStatisticsMatch(&lMatch);
 
-#if USE_GTK
+#if defined(USE_GTK)
     if (fX) {
         GTKDumpStatcontext(0);
         return;
@@ -1522,7 +1608,7 @@ CommandShowStatisticsGame(char *UNUSED(sz))
 {
 
     moverecord *pmr;
-    char szOutput[4096];
+    char szOutput[STATCONTEXT_MAXSIZE];
 
     if (!CheckGameExists())
         return;
@@ -1533,7 +1619,7 @@ CommandShowStatisticsGame(char *UNUSED(sz))
 
     g_assert(pmr->mt == MOVE_GAMEINFO);
 
-#if USE_GTK
+#if defined(USE_GTK)
     if (fX) {
         GTKDumpStatcontext(getGameNumber(plGame) + 1);
         return;
@@ -1569,7 +1655,7 @@ CommandAnalyseMove(char *UNUSED(sz))
         md.aamf = aamfAnalysis;
         RunAsyncProcess((AsyncFun) asyncAnalyzeMove, &md, _("Analysing move..."));
 
-#if USE_GTK
+#if defined(USE_GTK)
         if (fX)
             ChangeGame(NULL);
 #endif
@@ -1633,7 +1719,6 @@ updateStatisticsGame(const listOLD * plGame)
 {
 
     listOLD *pl;
-    moverecord *pmr;
     moverecord *pmrx = plGame->plNext->p;
     matchstate msAnalyse;
 
@@ -1641,7 +1726,7 @@ updateStatisticsGame(const listOLD * plGame)
 
     for (pl = plGame->plNext; pl != plGame; pl = pl->plNext) {
 
-        pmr = pl->p;
+        moverecord *pmr = pl->p;
 
         updateStatisticsMove(pmr, &msAnalyse, plGame, &pmrx->g.sc);
 
@@ -1699,10 +1784,10 @@ AnalyseClearMove(moverecord * pmr)
 
         pmr->CubeDecPtr->esDouble.et = pmr->esChequer.et = EVAL_NONE;
         pmr->n.stMove = pmr->stCube = SKILL_NONE;
-        pmr->rLuck = (float) ERR_VAL;
+        pmr->rLuck = ERR_VAL;
         pmr->lt = LUCK_NONE;
         if (pmr->ml.amMoves) {
-            free(pmr->ml.amMoves);
+            g_free(pmr->ml.amMoves);
             pmr->ml.amMoves = NULL;
         }
         pmr->ml.cMoves = 0;
@@ -1725,7 +1810,7 @@ AnalyseClearMove(moverecord * pmr)
     case MOVE_SETDICE:
 
         pmr->lt = LUCK_NONE;
-        pmr->rLuck = (float) ERR_VAL;
+        pmr->rLuck = ERR_VAL;
         break;
 
     default:
@@ -1757,7 +1842,7 @@ CommandAnalyseClearMove(char *UNUSED(sz))
 
     if (plLastMove && plLastMove->plNext && plLastMove->plNext->p) {
         AnalyseClearMove(plLastMove->plNext->p);
-#if USE_GTK
+#if defined(USE_GTK)
         if (fX)
             ChangeGame(NULL);
 #endif
@@ -1775,7 +1860,7 @@ CommandAnalyseClearGame(char *UNUSED(sz))
 
     AnalyseClearGame(plGame);
 
-#if USE_GTK
+#if defined(USE_GTK)
     if (fX)
         ChangeGame(NULL);
 #endif
@@ -1794,7 +1879,7 @@ CommandAnalyseClearMatch(char *UNUSED(sz))
     for (pl = lMatch.plNext; pl != &lMatch; pl = pl->plNext)
         AnalyseClearGame(pl->p);
 
-#if USE_GTK
+#if defined(USE_GTK)
     if (fX)
         ChangeGame(NULL);
 #endif
@@ -2029,7 +2114,7 @@ static void
 cmark_move_show(GString * gsz, const matchstate * UNUSED(pms), const moverecord * pmr, int movenr)
 {
     guint i;
-    gchar sz[40];
+    gchar sz[FORMATEDMOVESIZE];
     int found = 0;
 
     g_return_if_fail(pmr);
@@ -2040,7 +2125,7 @@ cmark_move_show(GString * gsz, const matchstate * UNUSED(pms), const moverecord
             if (!found++)
                 g_string_append_printf(gsz, _("Move %d\n"), movenr);
             FormatMove(sz, msBoard(), pmr->ml.amMoves[i].anMove);
-            g_string_append_printf(gsz, _("%i (%s) marked\n"), i + 1, sz);
+            g_string_append_printf(gsz, _("%u (%s) marked\n"), i + 1, sz);
         }
     }
 }
@@ -2068,10 +2153,8 @@ cmark_game_show(GString * gsz, listOLD * game, int game_number)
             ApplyMoveRecord(&ms_local, game, pmr);
             break;
         case MOVE_NORMAL:
-            if (pmr->fPlayer != ms_local.fMove) {
+            if (pmr->fPlayer != ms_local.fMove)
                 SwapSides(ms_local.anBoard);
-                ms_local.fMove = pmr->fPlayer;
-            }
             ms_local.fTurn = ms_local.fMove = pmr->fPlayer;
             cmark_cube_show(gsz, &ms_local, pmr, movenr);
             ms_local.anDice[0] = pmr->anDice[0];
@@ -2123,16 +2206,9 @@ cmark_move_set(moverecord * pmr, gchar * sz, CMark cmark)
 
     c = pmr->ml.cMoves;
 
-    /* Mike Petch - This is a hack and is not thread safe under GLIB version < 2.12.5
-     * Was added to allow cygwin to compile */
-
-#if GLIB_CHECK_VERSION(2,12,5)
     while ((n = (int) g_ascii_strtoll(sz, &sz, 10)) != 0) {
-#else
-    while ((n = (int) strtoll(sz, &sz, 10)) != 0) {
-#endif
         if (n > c) {
-            outputerrf("Only %d moves in movelist\n", c);
+            outputerrf(_("Only %d legal moves, cannot mark move %d\n"), c, n);
             g_slist_free(list);
             return;
         }
@@ -2140,7 +2216,7 @@ cmark_move_set(moverecord * pmr, gchar * sz, CMark cmark)
             list = g_slist_append(list, GINT_TO_POINTER(n));
     }
     if (g_slist_length(list) == 0) {
-        outputerrf("Not a valid list of moves\n");
+        outputerrf(_("Not a valid list of moves\n"));
         return;
     }
 
@@ -2207,18 +2283,17 @@ cmark_match_clear(listOLD * match)
 static int
 cmark_move_rollout(moverecord * pmr, gboolean destroy)
 {
-    gchar(*asz)[40];
+    gchar(*asz)[FORMATEDMOVESIZE];
     cubeinfo ci;
     cubeinfo **ppci;
     GSList *pl = NULL;
     gint c;
     guint j;
     gint res;
-    move *m;
     move **ppm;
     void *p;
     GSList *list = NULL;
-    positionkey key = { {0, 0, 0, 0, 0, 0} };
+    positionkey key = { {0, 0, 0, 0, 0, 0, 0} };
 
     g_return_val_if_fail(pmr, -1);
 
@@ -2233,20 +2308,20 @@ cmark_move_rollout(moverecord * pmr, gboolean destroy)
 
     ppm = g_new(move *, c);
     ppci = g_new(cubeinfo *, c);
-    asz = (char (*)[40]) g_malloc(40 * c);
+    asz = (char (*)[FORMATEDMOVESIZE]) g_malloc(FORMATEDMOVESIZE * c);
     if (pmr->n.iMove != UINT_MAX)
         CopyKey(pmr->ml.amMoves[pmr->n.iMove].key, key);
     GetMatchStateCubeInfo(&ci, &ms);
 
     for (pl = list, j = 0; pl; pl = g_slist_next(pl), j++) {
         gint i = GPOINTER_TO_INT(pl->data);
-        m = ppm[j] = &pmr->ml.amMoves[i];
+        move *m = ppm[j] = &pmr->ml.amMoves[i];
         ppci[j] = &ci;
         FormatMove(asz[j], msBoard(), m->anMove);
     }
 
     RolloutProgressStart(&ci, c, NULL, &rcRollout, asz, TRUE, &p);
-    ScoreMoveRollout(ppm, (const cubeinfo **) ppci, c, RolloutProgress, p);
+    ScoreMoveRollout(ppm, ppci, c, RolloutProgress, p);
     res = RolloutProgressEnd(&p, destroy);
 
     g_free(asz);
@@ -2262,7 +2337,7 @@ cmark_move_rollout(moverecord * pmr, gboolean destroy)
 
                 break;
             }
-#if USE_GTK
+#if defined(USE_GTK)
     if (fX)
         ChangeGame(NULL);
     else
@@ -2297,7 +2372,7 @@ cmark_cube_rollout(moverecord * pmr, gboolean destroy)
     float aarOutput[2][NUM_ROLLOUT_OUTPUTS];
     float aarStdDev[2][NUM_ROLLOUT_OUTPUTS];
     rolloutstat aarsStatistics[2][2];
-    gchar asz[2][40];
+    gchar asz[2][FORMATEDMOVESIZE];
     void *p;
     int res;
 
@@ -2323,7 +2398,7 @@ cmark_cube_rollout(moverecord * pmr, gboolean destroy)
 
     pmr->CubeDecPtr->esDouble.et = EVAL_ROLLOUT;
 
-#if USE_GTK
+#if defined(USE_GTK)
     if (fX)
         ChangeGame(NULL);
 #endif
@@ -2426,16 +2501,16 @@ check_cube_in_pmr(const moverecord * pmr)
     }
 
     if (pmr->mt != MOVE_NORMAL && pmr->mt != MOVE_DOUBLE && pmr->mt != MOVE_TAKE && pmr->mt != MOVE_DROP) {
-        outputerrf(_("This move doesn't imply a cubeaction. Cannot mark."));
+        outputerrf(_("This move doesn't imply a cube action. Cannot mark."));
         return 0;
     }
     return 1;
 }
 
-static gint
+static guint
 check_cmoves_in_pmr(const moverecord * pmr)
 {
-    gint c;
+    guint c;
 
     if (!pmr) {
         outputerrf(_("No moverecord stored for this move."));
@@ -2618,7 +2693,8 @@ CommandAnalyseRolloutMove(char *sz)
         cmark_move_set(pmr, sz, CMARK_ROLLOUT);
 
     if (cmark_move_rollout(pmr, FALSE) == 0) {
-        outputerrf("No moves marked for rollout\n");
+        outputerrf(_("No moves marked for rollout"));
+        outputerrf("\n");
         return;
     }
 
