@@ -1,14 +1,11 @@
 /*
- * dbprovider.c
+ * Copyright (C) 2008-2009 Jon Kinsey <jonkinsey@gmail.com>
+ * Copyright (C) 2008-2019 the AUTHORS
  *
- * by Jon Kinsey, 2008
- *
- * Database providers
- *
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 3 or later of the GNU General Public License as
- * published by the Free Software Foundation.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -16,10 +13,9 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
- * $Id: dbprovider.c,v 1.56 2018/07/31 20:15:11 plm Exp $
+ * $Id: dbprovider.c,v 1.64 2023/12/30 07:26:00 plm Exp $
  */
 
 #include "config.h"
@@ -68,28 +64,105 @@ static int SQLiteDeleteDatabase(const char *dbfilename, const char *user, const
 static GList *SQLiteGetDatabaseList(const char *user, const char *password, const char *hostname);
 static DBProvider providers[NUM_PROVIDERS] = {
 #if defined(USE_SQLITE)
-    {SQLiteConnect, SQLiteDisconnect, SQLiteSelect, SQLiteUpdateCommand, SQLiteCommit, SQLiteGetDatabaseList,
-     SQLiteDeleteDatabase,
-     "SQLite", "SQLite", "Direct SQLite3 connection", FALSE, TRUE, "gnubg", "", "", ""},
+    {
+	.Connect = SQLiteConnect,
+	.Disconnect = SQLiteDisconnect,
+	.Select = SQLiteSelect,
+	.UpdateCommand = SQLiteUpdateCommand,
+	.Commit = SQLiteCommit,
+	.GetDatabaseList = SQLiteGetDatabaseList,
+	.DeleteDatabase = SQLiteDeleteDatabase,
+	.name = "SQLite",
+	.shortname = "SQLite",
+	.desc = N_("Direct SQLite3 connection"),
+	.HasUserDetails = FALSE,
+	.storeGameStats = TRUE,
+	.database = NULL,
+	.username = NULL,
+	.password = NULL,
+	.hostname = NULL
+    },
 #endif
 #if defined(USE_PYTHON)
 #if !defined(USE_SQLITE)
-    {PySQLiteConnect, PyDisconnect, PySelect, PyUpdateCommand, PyCommit, SQLiteGetDatabaseList, SQLiteDeleteDatabase,
-     "SQLite (Python)", "PythonSQLite", "SQLite3 connection included in latest Python version", FALSE, TRUE, "gnubg",
-     "", "", ""},
+    {
+	.Connect = PySQLiteConnect,
+	.Disconnect = PyDisconnect,
+	.Select = PySelect,
+	.UpdateCommand = PyUpdateCommand,
+	.Commit = PyCommit,
+	.GetDatabaseList = SQLiteGetDatabaseList,
+	.DeleteDatabase = SQLiteDeleteDatabase,
+	.name = "SQLite (Python)",
+	.shortname = "PythonSQLite",
+	.desc = N_("SQLite3 connection via Python"),
+	.HasUserDetails = FALSE,
+	.storeGameStats = TRUE,
+	.database = NULL,
+	.username = NULL,
+	.password = NULL,
+	.hostname = NULL
+    },
 #endif
-    {PyMySQLConnect, PyDisconnect, PySelect, PyUpdateCommand, PyCommit, PyMySQLGetDatabaseList, PyMySQLDeleteDatabase,
-     "MySQL (Python)", "PythonMySQL", "MySQL connection via MySQLdb Python module", TRUE, TRUE, "gnubg", "", "",
-     "localhost:3306"},
-    {PyPostgreConnect, PyDisconnect, PySelect, PyUpdateCommand, PyCommit, PyPostgreGetDatabaseList,
-     PyPostgreDeleteDatabase,
-     "PostgreSQL (Python)", "PythonPostgre", "PostgreSQL connection via PyGreSQL Python module", TRUE, TRUE, "gnubg", "",
-     "", "localhost:5432"},
+    {
+	.Connect = PyMySQLConnect,
+	.Disconnect = PyDisconnect,
+	.Select = PySelect,
+	.UpdateCommand = PyUpdateCommand,
+	.Commit = PyCommit,
+	.GetDatabaseList = PyMySQLGetDatabaseList,
+	.DeleteDatabase = PyMySQLDeleteDatabase,
+	.name = "MySQL (Python)",
+	.shortname = "PythonMySQL",
+	.desc = N_("MySQL/MariaDB connection via MySQLdb Python module"),
+	.HasUserDetails = TRUE,
+	.storeGameStats = TRUE,
+	.database = NULL,
+	.username = NULL,
+	.password = NULL,
+	.hostname = NULL
+    },
+    {
+	.Connect = PyPostgreConnect,
+	.Disconnect = PyDisconnect,
+	.Select = PySelect,
+	.UpdateCommand = PyUpdateCommand,
+	.Commit = PyCommit,
+	.GetDatabaseList = PyPostgreGetDatabaseList,
+	.DeleteDatabase = PyPostgreDeleteDatabase,
+	.name = "PostgreSQL (Python)",
+	.shortname = "PythonPostgre",
+	.desc = N_("PostgreSQL connection via PyGreSQL Python module"),
+	.HasUserDetails = TRUE,
+	.storeGameStats = TRUE,
+	.database = NULL,
+	.username = NULL,
+	.password = NULL,
+	.hostname = NULL
+    }
 #endif
 };
-
 #else
-DBProvider providers[1] = { {0, 0, 0, 0, 0, 0, 0, "No Providers", "No Providers", "No Providers", 0, 0, 0, 0, 0, 0} };
+DBProvider providers[1] = {
+    {
+	.Connect = NULL,
+	.Disconnect = NULL,
+	.Select = NULL,
+	.UpdateCommand = NULL,
+	.Commit = NULL,
+	.GetDatabaseList = NULL,
+	.DeleteDatabase = NULL,
+	.name = "No Providers",
+	.shortname = "No Providers",
+	.desc = N_("No database providers"),
+	.HasUserDetails = FALSE,
+	.storeGameStats = FALSE,
+	.database = NULL,
+	.username = NULL,
+	.password = NULL,
+	.hostname = NULL
+    }
+};
 #endif
 
 #if defined(USE_PYTHON) || defined(USE_SQLITE)
@@ -97,15 +170,14 @@ static RowSet *
 MallocRowset(size_t rows, size_t cols)
 {
     size_t i;
-    RowSet *pRow = malloc(sizeof(RowSet));
-    g_assert(pRow);
+    RowSet *pRow = g_malloc(sizeof(RowSet));
 
-    pRow->widths = (size_t *) malloc(cols * sizeof(size_t));
+    pRow->widths = (size_t *) g_malloc(cols * sizeof(size_t));
     memset(pRow->widths, 0, cols * sizeof(size_t));
 
-    pRow->data = malloc(rows * sizeof(char **));
+    pRow->data = g_malloc(rows * sizeof(char **));
     for (i = 0; i < rows; i++) {
-        pRow->data[i] = malloc(cols * sizeof(char *));
+        pRow->data[i] = g_malloc(cols * sizeof(char *));
         memset(pRow->data[i], 0, cols * sizeof(char *));
     }
 
@@ -122,8 +194,7 @@ SetRowsetData( /*lint -e{818} */ RowSet * rs, size_t row, size_t col, const char
     if (!data)
         data = "";
 
-    rs->data[row][col] = malloc(strlen(data) + 1);
-    strcpy(rs->data[row][col], data);
+    rs->data[row][col] = g_strdup(data);
 
     size = strlen(data);
     if (row == 0 || size > rs->widths[col])
@@ -135,23 +206,23 @@ extern void
 FreeRowset(RowSet * pRow)
 {
     if (pRow != NULL) {
-        free(pRow->widths);
+        g_free(pRow->widths);
 
         if (pRow->data != NULL) {
             unsigned int i, j;
 
             for (i = 0; i < pRow->rows; i++) {
                 for (j = 0; j < pRow->cols; j++) {
-                    free(pRow->data[i][j]);
+                    g_free(pRow->data[i][j]);
                 }
-            free(pRow->data[i]);
+            g_free(pRow->data[i]);
             }
         }
 
-        free(pRow->data);
+        g_free(pRow->data);
     }   
 
-    free(pRow);
+    g_free(pRow);
 }
 
 int
@@ -201,6 +272,30 @@ GetTypeFromName(const char *name)
     return (DBProviderType) i;
 }
 
+void
+DefaultDBSettings(void)
+{
+#if NUM_PROVIDERS
+    int i = 0;
+#if defined(USE_SQLITE)
+    /* native sqlite3 */
+    providers[i++].database = g_strdup("gnubg");
+#endif
+#if defined(USE_PYTHON)
+#if !defined(USE_SQLITE)
+    /* sqlite3 via Python */
+    providers[i++].database = g_strdup("gnubg");
+#endif
+    /* MySQL / MariaDB */
+    providers[i].hostname = g_strdup("localhost:3306");
+    providers[i++].database = g_strdup("gnubg");
+    /* PostgreSQL */
+    providers[i].hostname = g_strdup("localhost:5432");
+    providers[i++].database = g_strdup("gnubg");
+#endif
+#endif
+}
+
 void
 SetDBParam(const char *db, const char *key, const char *value)
 {
@@ -208,14 +303,19 @@ SetDBParam(const char *db, const char *key, const char *value)
     if (type == INVALID_PROVIDER)
         return;
 
-    if (!StrCaseCmp(key, "database"))
-        providers[type].database = g_strdup(value);
-    else if (!StrCaseCmp(key, "username"))
+    if (!StrCaseCmp(key, "database")) {
+        g_free(providers[type].database);
+        providers[type].database= g_strdup(value);
+    } else if (!StrCaseCmp(key, "username")) {
+        g_free(providers[type].username);
         providers[type].username = g_strdup(value);
-    else if (!StrCaseCmp(key, "password"))
+    } else if (!StrCaseCmp(key, "password")) {
+        g_free(providers[type].password);
         providers[type].password = g_strdup(value);
-    else if (!StrCaseCmp(key, "hostname"))
-        providers[type].hostname = g_strdup(value);
+    } else if (!StrCaseCmp(key, "hostname")) {
+        g_free(providers[type].hostname);
+	providers[type].hostname = g_strdup(value);
+    }
 }
 
 void
@@ -228,9 +328,17 @@ void
 SetDBSettings(DBProviderType dbType, const char *database, const char *user, const char *password, const char *hostname)
 {
     dbProviderType = dbType;
+    
+    g_free(providers[dbProviderType].database);
     providers[dbProviderType].database = g_strdup(database);
+
+    g_free(providers[dbProviderType].username);
     providers[dbProviderType].username = g_strdup(user);
+
+    g_free(providers[dbProviderType].password);
     providers[dbProviderType].password = g_strdup(password);
+
+    g_free(providers[dbProviderType].hostname);
     providers[dbProviderType].hostname = g_strdup(hostname);
 }
 
@@ -462,9 +570,9 @@ ConvertPythonToRowset(PyObject * v)
                     continue;
                 }
                 if (PyUnicode_Check(e2))
-                    strcpy(buf, PyBytes_AsString(PyUnicode_AsUTF8String(e2)));
+                    g_strlcpy(buf, PyBytes_AsString(PyUnicode_AsUTF8String(e2)), sizeof(buf));
                 else if (PyBytes_Check(e2))
-                    strcpy(buf, PyBytes_AsString(e2));
+                    g_strlcpy(buf, PyBytes_AsString(e2), sizeof(buf));
                 else if (PyInt_Check(e2) || PyLong_Check(e2)
                          || !StrCaseCmp(e2->ob_type->tp_name, "Decimal"))       /* Not sure how to check for decimal type directly */
                     sprintf(buf, "%ld", PyInt_AsLong(e2));
@@ -605,11 +713,15 @@ RowSet *
 SQLiteSelect(const char *str)
 {
     int ret;
-    char *buf = g_strdup_printf("Select %s;", str);
+    char *buf = g_strdup_printf("SELECT %s;", str);
     RowSet *rs = NULL;
 
     sqlite3_stmt *pStmt;
+#if SQLITE_VERSION_NUMBER >= 3003011
+    ret = sqlite3_prepare_v2(connection, buf, -1, &pStmt, NULL);
+#else
     ret = sqlite3_prepare(connection, buf, -1, &pStmt, NULL);
+#endif
     g_free(buf);
     if (ret == SQLITE_OK) {
         size_t i, row;
@@ -618,8 +730,13 @@ SQLiteSelect(const char *str)
 
         while ((ret = sqlite3_step(pStmt)) == SQLITE_ROW)
             numRows++;
+
+        if (ret != SQLITE_DONE)
+            outputerrf("SQL error: %s in sqlite3_step()", sqlite3_errmsg(connection));
+
         if (sqlite3_reset(pStmt) != SQLITE_OK)
             outputerrf("SQL error: %s in sqlite3_reset()", sqlite3_errmsg(connection));
+
         rs = MallocRowset(numRows + 1, numCols);        /* first row is headings */
 
         for (i = 0; i < numCols; i++)
