@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2005 Ingo Macherius
- * Copyright (C) 2005-2019 the AUTHORS
+ * Copyright (C) 2005-2023 the AUTHORS
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -15,7 +15,7 @@
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
- * $Id: gtkfile.c,v 1.76 2022/09/06 21:02:05 plm Exp $
+ * $Id: gtkfile.c,v 1.83 2023/12/17 17:58:31 plm Exp $
  */
 
 #include "config.h"
@@ -38,14 +38,26 @@
 #include "file.h"
 #include "util.h"
 
+/*for picking the first file in folder...*/
+#include <stdio.h>
+#include <dirent.h>
+#include <sys/stat.h>
+#include <unistd.h>
+#include <time.h>
+#include <sys/types.h>
+
+#define MAX_LEN 1024
 
 static void
 FilterAdd(const char *fn, const char *pt, GtkFileChooser * fc)
 {
     GtkFileFilter *aff = gtk_file_filter_new();
+    gchar *sz;
+
     gtk_file_filter_set_name(aff, fn);
     gtk_file_filter_add_pattern(aff, pt);
-    gtk_file_filter_add_pattern(aff, g_ascii_strup(pt, -1));
+    gtk_file_filter_add_pattern(aff, sz = g_ascii_strup(pt, -1));
+    g_free(sz);
     gtk_file_chooser_add_filter(fc, aff);
 }
 
@@ -281,10 +293,15 @@ static void
 selection_changed_cb(GtkFileChooser * file_chooser, void *UNUSED(notused))
 {
     const char *label;
-    char *buf;
-    char *filename = gtk_file_chooser_get_filename(file_chooser);
-    FilePreviewData *fpd = ReadFilePreview(filename);
+    gchar *buf;
+    gchar *filename;
+    FilePreviewData *fpd;
     int openable = FALSE;
+
+    filename = gtk_file_chooser_get_filename(file_chooser);
+    fpd = ReadFilePreview(filename);
+    g_free(filename);
+
     if (!fpd) {
         lastOpenType = N_IMPORT_TYPES;
         label = "";
@@ -307,13 +324,14 @@ add_import_filters(GtkFileChooser * fc)
 {
     GtkFileFilter *aff = gtk_file_filter_new();
     gint i;
-    gchar *sg;
+    gchar *sg, *sz;
 
     gtk_file_filter_set_name(aff, _("Supported files"));
     for (i = 0; i < N_IMPORT_TYPES; ++i) {
         sg = g_strdup_printf("*%s", import_format[i].extension);
         gtk_file_filter_add_pattern(aff, sg);
-        gtk_file_filter_add_pattern(aff, g_ascii_strup(sg, -1));
+        gtk_file_filter_add_pattern(aff, sz = g_ascii_strup(sg, -1));
+        g_free(sz);
         g_free(sg);
     }
     gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(fc), aff);
@@ -494,7 +512,17 @@ batch_create_save(gchar * filename, gchar ** save, char **result)
     gchar *file;
     gchar *folder;
     gchar *dir;
+
     DisectPath(filename, NULL, &file, &folder);
+    
+    if (file == NULL || folder == NULL) {
+	g_free(file);
+	g_free(folder);
+	if (result)
+            *result = _("Incorrect path");
+	return FALSE;
+    }
+ 
     dir = g_build_filename(folder, "analysed", NULL);
     g_free(folder);
 
@@ -502,15 +530,17 @@ batch_create_save(gchar * filename, gchar ** save, char **result)
         g_mkdir(dir, 0700);
 
     if (!g_file_test(dir, G_FILE_TEST_IS_DIR)) {
+        g_free(file);
         g_free(dir);
         if (result)
-            *result = _("Failed to make directory");
+            *result = _("Failed to create directory");
         return FALSE;
     }
 
     *save = g_strconcat(dir, G_DIR_SEPARATOR_S, file, ".sgf", NULL);
     g_free(file);
     g_free(dir);
+
     return TRUE;
 }
 
@@ -527,12 +557,11 @@ batch_analyse(gchar * filename, char **result, gboolean add_to_db, gboolean add_
     printf("save %s\n", save);
 
     if (g_file_test((save), G_FILE_TEST_EXISTS)) {
-        *result = _("Previous");
+        *result = _("Pre-existing");
         g_free(save);
         return TRUE;
     }
 
-
     g_free(szCurrentFileName);
     szCurrentFileName = NULL;
     cmd = g_strdup_printf("import auto \"%s\"", filename);
@@ -587,7 +616,7 @@ batch_do_all(gpointer batch_model, gboolean add_to_db, gboolean add_incdata_to_d
         gchar *filename;
         gint cancelled;
         gtk_tree_model_get(batch_model, &iter, COL_PATH, &filename, -1);
-
+        //outputerrf("filename=%s\n", filename);
         batch_analyse(filename, &result, add_to_db, add_incdata_to_db);
         gtk_list_store_set(batch_model, &iter, COL_RESULT, result, -1);
 
@@ -673,7 +702,7 @@ batch_create_view(GSList * filenames)
     gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(view),
                                                 -1, _("Result"), renderer, "text", COL_RESULT, NULL);
 
-    gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(view), -1, _("Type"), renderer, "text", COL_DESC, NULL);
+    gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(view), -1, Q_("fileType|Type"), renderer, "text", COL_DESC, NULL);
 
     gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(view), -1, _("File"), renderer, "text", COL_FILE, NULL);
     model = batch_create_model(filenames);
@@ -792,6 +821,175 @@ batch_create_dialog_and_run(GSList * filenames, gboolean add_to_db)
     gtk_window_set_modal(GTK_WINDOW(dialog), FALSE);
 }
 
+extern void
+GTKAnalyzeCurrent(void)
+{
+    /*analyze match*/
+    UserCommand("analyse match");
+    if (fAutoDB) {
+        /*add match to db*/
+        CommandRelationalAddMatch(NULL);
+    }
+    /*show stats panel*/
+    UserCommand("show statistics match");
+    return;
+}
+
+/* functions to find latest file in folder */
+
+static void recentByModification(const char* path, char* recent){
+    char buffer[MAX_LEN];
+    FilePreviewData *fdp;
+    struct dirent* entry;
+    time_t recenttime = 0;
+    struct stat statbuf;
+    DIR* dir  = opendir(path);
+
+    if (dir) {
+        while (NULL != (entry = readdir(dir))) {
+           // outputerrf(_("`%s' ... looking at file....: %s, time: %lld"), entry->d_name, recent, (long long)recenttime);
+
+            /* we first check that it's a file*/
+            if(strcmp(entry->d_name, ".") == 0 || strcmp(entry->d_name, "..") == 0 || strncmp(entry->d_name, "/", 1) == 0 || strncmp(entry->d_name, "\\", 1) == 0)
+                continue;
+            /* check file type when DIRENT is defined; could use stat if not*/
+#ifdef _DIRENT_HAVE_D_TYPE
+            if (entry->d_type == DT_REG) 
+// #else
+//             DIR* dir2 = opendir(entry);
+//             if(dir2==NULL) {
+#endif 
+            {    
+                /* we then check that it's more recent than what we've seen so far*/
+                sprintf(buffer, "%s/%s", path, entry->d_name);
+                stat(buffer, &statbuf);            
+                if (statbuf.st_mtime > recenttime) {
+                    /* next we check that it's a correct file format*/
+                    fdp = ReadFilePreview(buffer);
+                    if (!fdp) {
+                        //outputerrf(_("`%s' is not a backgammon file (especially %s)... looking at file....: %s, time: %lld"), buffer,entry->d_name, recent, (long long) recenttime);
+                        g_free(fdp);
+                        continue;
+                    } else if (fdp->type == N_IMPORT_TYPES) {
+                        //outputerrf(_("The format of '%s' is not recognized  (especially %s)"), buffer,entry->d_name);
+                        g_free(fdp);
+                        continue;
+                    } else {
+                        /* finally it passed all the checks, we keep it for now in the char * recent */
+                        strncpy(recent, buffer, MAX_LEN);
+                        // strncpy(recent, entry->d_name, MAX_LEN);
+                        recenttime = statbuf.st_mtime;
+                        // g_message("looking at file....: %s, time: %lld\n", recent, (long long) recenttime);
+                        g_free(fdp);
+                    }
+                }
+            }
+        }
+        closedir(dir);
+    } else {
+        outputerrf("Unable to read the directory");
+    }
+}
+
+static void
+AnalyzeSingleFile(void)
+{
+    gchar *folder = NULL;
+    gchar *filename = NULL;
+    GtkWidget *fc;
+    static gchar *last_folder = NULL;
+
+    folder = last_folder ? last_folder : default_import_folder;
+
+    /* now select a file; could also use GTKFileSelect() */
+    fc = GnuBGFileDialog(_("Select file to analyse"), folder, NULL, GTK_FILE_CHOOSER_ACTION_OPEN);
+    gtk_file_chooser_set_select_multiple(GTK_FILE_CHOOSER(fc), FALSE);
+    add_import_filters(GTK_FILE_CHOOSER(fc));
+
+    if (gtk_dialog_run(GTK_DIALOG(fc)) == GTK_RESPONSE_ACCEPT) {
+        filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(fc));
+    }
+    if (filename) {
+        gchar* cmd;
+
+        g_free(last_folder);
+        last_folder = gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER(fc));
+        gtk_widget_destroy(fc);
+
+        // char buffer[MAX_LEN];
+        // sprintf(buffer, "%s/%s", last_folder, filename);
+        // // if (!get_input_discard())
+        // //     return;
+
+        /*open this file*/
+        cmd = g_strdup_printf("import auto \"%s\"", filename);
+        UserCommand(cmd);
+        g_free(cmd);
+        ////outputerrf("filename=%s\n", filename);
+        //CommandImportAuto(filename);
+        g_free(filename);
+
+        /*analyze match*/
+        UserCommand("analyse match");
+        if(fAutoDB) {
+            /*add match to db*/
+            CommandRelationalAddMatch(NULL);
+        }
+        /*show stats panel*/
+        UserCommand("show statistics match");
+        return;
+
+    } else
+        gtk_widget_destroy(fc);
+}
+
+
+static void
+SmartAnalyze(void)
+{
+    gchar *folder = NULL;
+    gchar *cmd;
+    char recent[MAX_LEN] = "";
+
+    folder = default_import_folder ? default_import_folder : ".";
+    // g_message("folder=%s\n", folder);
+
+    /* find most recent file in the folder and write its name (in char recent[])*/
+    recentByModification(folder, recent);
+
+    /*open this file*/
+    cmd = g_strdup_printf("import auto \"%s\"", recent);
+    UserCommand(cmd);
+    g_free(cmd);
+
+    //outputerrf("recent=%s\n", recent);
+    //CommandImportAuto(recent);
+
+    /*analyze match*/
+    UserCommand("analyse match");
+    if(fAutoDB) {
+        /*add match to db*/
+        CommandRelationalAddMatch(NULL);
+    }
+    /*show stats panel*/
+    UserCommand("show statistics match");
+    return;
+}
+
+extern void
+GTKAnalyzeFile(void)
+{
+    // g_message("GTKAnalyzeFile(): %d\n", AnalyzeFileSettingDef);
+    if (AnalyzeFileSettingDef == AnalyzeFileBatch) {
+        GTKBatchAnalyse(NULL, 0, NULL);
+    } else if (AnalyzeFileSettingDef == AnalyzeFileRegular) {
+        AnalyzeSingleFile();
+    } else { //   AnalyzeFileSmart, 
+        SmartAnalyze();
+    }
+    return;
+}
+
 extern void
 GTKBatchAnalyse(gpointer UNUSED(p), guint UNUSED(n), GtkWidget * UNUSED(pw))
 {
@@ -809,7 +1007,7 @@ GTKBatchAnalyse(gpointer UNUSED(p), guint UNUSED(n), GtkWidget * UNUSED(pw))
     add_import_filters(GTK_FILE_CHOOSER(fc));
 
     add_to_db = gtk_check_button_new_with_label(_("Add to database"));
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(add_to_db), TRUE);
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(add_to_db), fAutoDB);
 
     gtk_file_chooser_set_extra_widget(GTK_FILE_CHOOSER(fc), add_to_db);
 
@@ -834,5 +1032,4 @@ GTKBatchAnalyse(gpointer UNUSED(p), guint UNUSED(n), GtkWidget * UNUSED(pw))
         fConfirmNew = fConfirmNew_s;
     } else
         gtk_widget_destroy(fc);
-
 }
