@@ -1,13 +1,11 @@
 /*
- * gtkpanels.c
- * by Jon Kinsey, 2004
+ * Copyright (C) 2004-2019 Jon Kinsey <jonkinsey@gmail.com>
+ * Copyright (C) 2005-2018 the AUTHORS
  *
- * Panels/window code
- *
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 3 or later of the GNU General Public License as
- * published by the Free Software Foundation.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,10 +13,9 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
- * $Id: gtkpanels.c,v 1.88 2018/07/21 09:18:12 plm Exp $
+ * $Id: gtkpanels.c,v 1.98 2023/12/20 14:17:30 plm Exp $
  */
 
 #include "config.h"
@@ -33,7 +30,7 @@
 #include "gtktoolbar.h"
 #include "positionid.h"
 #if defined(USE_BOARD3D)
-#include "fun3d.h"
+#include "inc3d.h"
 #endif
 
 static int fDisplayPanels = TRUE;
@@ -42,7 +39,7 @@ static int fDockPanels = TRUE;
 #define NUM_CMD_HISTORY 10
 #define KEY_TAB -247
 
-typedef struct _CommandEntryData_T {
+typedef struct {
     GtkWidget *pwEntry, *pwHelpText, *cmdEntryCombo;
     int showHelp;
     int numHistory;
@@ -74,7 +71,7 @@ static gboolean ShowAnalysis(void);
 static gboolean ShowTheoryWindow(void);
 static gboolean ShowCommandWindow(void);
 
-typedef struct _windowobject {
+typedef struct {
     const char *winName;
     int showing;
     int docked;
@@ -102,7 +99,7 @@ static windowobject woPanel[NUM_WINDOWS] = {
      TRUE, TRUE, TRUE, TRUE,
      ShowGameWindow, DeleteGame,
      0,
-     {250, 200, 20, 20, FALSE}
+     {400, 200, 20, 100, FALSE}
      },
     /* analysis */
     {
@@ -110,7 +107,7 @@ static windowobject woPanel[NUM_WINDOWS] = {
      TRUE, TRUE, TRUE, TRUE,
      ShowAnalysis, DeleteAnalysis,
      0,
-     {0, 400, 20, 20, FALSE}
+     {600, 300, 40, 120, FALSE}
      },
     /* annotation */
     {
@@ -118,7 +115,7 @@ static windowobject woPanel[NUM_WINDOWS] = {
      FALSE, TRUE, TRUE, FALSE,
      ShowAnnotation, DeleteAnnotation,
      0,
-     {0, 400, 20, 20, FALSE}
+     {400, 200, 50, 50, FALSE}
      },
     /* hint */
     {
@@ -134,7 +131,7 @@ static windowobject woPanel[NUM_WINDOWS] = {
      FALSE, TRUE, TRUE, TRUE,
      ShowMessage, DeleteMessage,
      0,
-     {0, 500, 20, 20, FALSE}
+     {300, 200, 20, 20, FALSE}
      },
     /* command */
     {
@@ -150,7 +147,7 @@ static windowobject woPanel[NUM_WINDOWS] = {
      FALSE, TRUE, TRUE, TRUE,
      ShowTheoryWindow, DeleteTheoryWindow,
      0,
-     {0, 0, 20, 20, FALSE}
+     {160, 0, 20, 20, FALSE}
      }
 };
 
@@ -174,7 +171,7 @@ extern gboolean
 ShowGameWindow(void)
 {
     ShowPanel(WINDOW_GAME);
-#if defined(USE_GTKUIMANAGER)
+#if !defined(USE_GTKITEMFACTORY)
     gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(gtk_ui_manager_get_widget(puim,
                                                                                  "/MainMenu/ViewMenu/PanelsMenu/GameRecord")),
                                    TRUE);
@@ -188,7 +185,7 @@ ShowGameWindow(void)
 static gboolean
 ShowAnnotation(void)
 {
-#if defined(USE_GTKUIMANAGER)
+#if !defined(USE_GTKITEMFACTORY)
     gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(gtk_ui_manager_get_widget(puim,
                                                                                  "/MainMenu/ViewMenu/PanelsMenu/Commentary")),
                                    TRUE);
@@ -200,7 +197,12 @@ ShowAnnotation(void)
     woPanel[WINDOW_ANNOTATION].showing = TRUE;
     /* Avoid showing before main window */
     if (gtk_widget_get_realized(pwMain))
-        gtk_widget_show_all(woPanel[WINDOW_ANNOTATION].pwWin);
+        /* FIXME?
+         * When undocked, commentary is not an independent
+         * window but the bottom pane of the analysis
+         */
+        if (woPanel[WINDOW_ANNOTATION].pwWin != NULL)
+            gtk_widget_show_all(woPanel[WINDOW_ANNOTATION].pwWin);
     return TRUE;
 }
 
@@ -208,7 +210,7 @@ static gboolean
 ShowMessage(void)
 {
     ShowPanel(WINDOW_MESSAGE);
-#if defined(USE_GTKUIMANAGER)
+#if !defined(USE_GTKITEMFACTORY)
     gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(gtk_ui_manager_get_widget(puim,
                                                                                  "/MainMenu/ViewMenu/PanelsMenu/Message")),
                                    TRUE);
@@ -222,7 +224,7 @@ static gboolean
 ShowAnalysis(void)
 {
     ShowPanel(WINDOW_ANALYSIS);
-#if defined(USE_GTKUIMANAGER)
+#if !defined(USE_GTKITEMFACTORY)
     gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(gtk_ui_manager_get_widget(puim,
                                                                                  "/MainMenu/ViewMenu/PanelsMenu/Analysis")),
                                    TRUE);
@@ -238,7 +240,7 @@ static gboolean
 ShowTheoryWindow(void)
 {
     ShowPanel(WINDOW_THEORY);
-#if defined(USE_GTKUIMANAGER)
+#if !defined(USE_GTKITEMFACTORY)
     gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(gtk_ui_manager_get_widget(puim,
                                                                                  "/MainMenu/ViewMenu/PanelsMenu/Theory")),
                                    TRUE);
@@ -252,7 +254,7 @@ static gboolean
 ShowCommandWindow(void)
 {
     ShowPanel(WINDOW_COMMAND);
-#if defined(USE_GTKUIMANAGER)
+#if !defined(USE_GTKITEMFACTORY)
     gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(gtk_ui_manager_get_widget(puim,
                                                                                  "/MainMenu/ViewMenu/PanelsMenu/Command")),
                                    TRUE);
@@ -342,9 +344,10 @@ UpdateTheoryData(BoardData * bd, int UpdateType, const TanBoard points)
     }
 
     if (UpdateType & TT_RETURNHITS) {
-        TanBoard anBoard;
         pc = NULL;
         if (bd->valid_move) {
+            TanBoard anBoard;
+
             PositionFromKey(anBoard, &bd->valid_move->key);
             pc = ReturnHits(anBoard);
         }
@@ -418,8 +421,7 @@ ShowHelp(GtkWidget * pwText, char *pStr)
     GtkTextIter iter;
 
     /* Copy string as token striping corrupts string */
-    pTemp = malloc(strlen(pStr) + 1);
-    strcpy(pTemp, pStr);
+    pTemp = strdup(pStr);
     cc = CheckCommand(pTemp, acTop);
 
     buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(pwText));
@@ -443,10 +445,10 @@ ShowHelp(GtkWidget * pwText, char *pStr)
                 }
             }
 
-            sprintf(szBuf, "Command: %s\n", szCommand);
+            sprintf(szBuf, "%s %s\n", _("Command:"), szCommand);
             gtk_text_buffer_insert(buffer, &iter, szBuf, -1);
             gtk_text_buffer_insert(buffer, &iter, gettext(pc->szHelp), -1);
-            sprintf(szBuf, "\n\nUsage: %s", szUsage);
+            sprintf(szBuf, "\n\n%s %s", _("Usage:"), szUsage);
             gtk_text_buffer_insert(buffer, &iter, szBuf, -1);
 
             if (!(pc->pc && pc->pc->sz))
@@ -510,8 +512,7 @@ CommandOK(GtkWidget * UNUSED(pw), CommandEntryData_T * pData)
     for (i = pData->numHistory; i > 0; i--)
         pData->cmdHistory[i] = pData->cmdHistory[i - 1];
 
-    pData->cmdHistory[0] = malloc(strlen(pData->cmdString) + 1);
-    strcpy(pData->cmdHistory[0], pData->cmdString);
+    pData->cmdHistory[0] = strdup(pData->cmdString);
     pData->numHistory++;
     PopulateCommandHistory(pData);
     gtk_entry_set_text(GTK_ENTRY(pData->pwEntry), "");
@@ -683,6 +684,7 @@ CreateAnalysisWindow(void)
         gtk_window_add_accel_group(GTK_WINDOW(woPanel[WINDOW_ANALYSIS].pwWin), pagMain);
 
         gtk_paned_pack1(GTK_PANED(pwPaned), pwAnalysis = gtk_label_new(NULL), TRUE, FALSE);
+
 #if GTK_CHECK_VERSION(3,0,0)
         pHbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
 #else
@@ -713,7 +715,7 @@ CreateAnalysisWindow(void)
     gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
     gtk_container_add(GTK_CONTAINER(sw), pwCommentary);
     gtk_box_pack_start(GTK_BOX(pHbox), sw, TRUE, TRUE, 0);
-    gtk_widget_set_size_request(sw, 100, 150);
+    gtk_widget_set_size_request(sw, 100, 50);
     buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(pwCommentary));
     g_signal_connect(G_OBJECT(buffer), "changed", G_CALLBACK(CommentaryChanged), buffer);
 
@@ -868,7 +870,7 @@ CreateHeadWindow(gnubgwindow panel, const char *sz, GtkWidget * pwWidge)
     pwHbox = gtk_hbox_new(FALSE, 0);
 #endif
 
-    gtk_button_set_image(GTK_BUTTON(pwX), gtk_image_new_from_stock(GTK_STOCK_CLOSE, GTK_ICON_SIZE_MENU));
+    gtk_button_set_image(GTK_BUTTON(pwX), gtk_image_new_from_icon_name("window-close", GTK_ICON_SIZE_MENU));
     g_signal_connect(G_OBJECT(pwX), "clicked", G_CALLBACK(woPanel[panel].hideFun), NULL);
 
     gtk_box_pack_start(GTK_BOX(pwVbox), pwHbox, FALSE, FALSE, 0);
@@ -904,7 +906,7 @@ static gboolean
 DeleteMessage(void)
 {
     HidePanel(WINDOW_MESSAGE);
-#if defined(USE_GTKUIMANAGER)
+#if !defined(USE_GTKITEMFACTORY)
     gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(gtk_ui_manager_get_widget(puim,
                                                                                  "/MainMenu/ViewMenu/PanelsMenu/Message")),
                                    FALSE);
@@ -919,7 +921,7 @@ static gboolean
 DeleteAnalysis(void)
 {
     HidePanel(WINDOW_ANALYSIS);
-#if defined(USE_GTKUIMANAGER)
+#if !defined(USE_GTKITEMFACTORY)
     gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(gtk_ui_manager_get_widget(puim,
                                                                                  "/MainMenu/ViewMenu/PanelsMenu/Analysis")),
                                    FALSE);
@@ -934,7 +936,7 @@ static gboolean
 DeleteAnnotation(void)
 {
     HidePanel(WINDOW_ANNOTATION);
-#if defined(USE_GTKUIMANAGER)
+#if !defined(USE_GTKITEMFACTORY)
     gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(gtk_ui_manager_get_widget(puim,
                                                                                  "/MainMenu/ViewMenu/PanelsMenu/Commentary")),
                                    FALSE);
@@ -949,7 +951,7 @@ static gboolean
 DeleteGame(void)
 {
     HidePanel(WINDOW_GAME);
-#if defined(USE_GTKUIMANAGER)
+#if !defined(USE_GTKITEMFACTORY)
     gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(gtk_ui_manager_get_widget(puim,
                                                                                  "/MainMenu/ViewMenu/PanelsMenu/GameRecord")),
                                    FALSE);
@@ -964,7 +966,7 @@ static gboolean
 DeleteTheoryWindow(void)
 {
     HidePanel(WINDOW_THEORY);
-#if defined(USE_GTKUIMANAGER)
+#if !defined(USE_GTKITEMFACTORY)
     gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(gtk_ui_manager_get_widget(puim,
                                                                                  "/MainMenu/ViewMenu/PanelsMenu/Theory")),
                                    FALSE);
@@ -978,7 +980,7 @@ static gboolean
 DeleteCommandWindow(void)
 {
     HidePanel(WINDOW_COMMAND);
-#if defined(USE_GTKUIMANAGER)
+#if !defined(USE_GTKITEMFACTORY)
     gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(gtk_ui_manager_get_widget(puim,
                                                                                  "/MainMenu/ViewMenu/PanelsMenu/Command")),
                                    FALSE);
@@ -1085,7 +1087,7 @@ DockPanels(void)
     if (fDockPanels) {
         RefreshGeometries();    /* Get the current window positions */
 
-#if defined(USE_GTKUIMANAGER)
+#if !defined(USE_GTKITEMFACTORY)
         gtk_widget_show((gtk_ui_manager_get_widget(puim, "/MainMenu/ViewMenu/PanelsMenu/Commentary")));
 
         if (fDisplayPanels) {
@@ -1120,7 +1122,7 @@ DockPanels(void)
         if (fDisplayPanels)
             SwapBoardToPanel(FALSE, TRUE);
 
-#if defined(USE_GTKUIMANAGER)
+#if !defined(USE_GTKITEMFACTORY)
         gtk_widget_hide((gtk_ui_manager_get_widget(puim, "/MainMenu/ViewMenu/PanelsMenu/Commentary")));
         gtk_widget_hide((gtk_ui_manager_get_widget(puim, "/MainMenu/ViewMenu/HidePanels")));
         gtk_widget_hide((gtk_ui_manager_get_widget(puim, "/MainMenu/ViewMenu/RestorePanels")));
@@ -1143,7 +1145,7 @@ DockPanels(void)
         CreateTheoryWindow();
         CreateCommandWindow();
     }
-#if defined(USE_GTKUIMANAGER)
+#if !defined(USE_GTKITEMFACTORY)
     gtk_widget_set_sensitive(gtk_ui_manager_get_widget(puim,
                                                        "/MainMenu/ViewMenu/PanelsMenu/Message"), !fDockPanels
                              || fDisplayPanels);
@@ -1180,7 +1182,7 @@ DockPanels(void)
         GTKSetGame(currentSelectedGame);
 
     /* Make sure check item is correct */
-#if defined(USE_GTKUIMANAGER)
+#if !defined(USE_GTKITEMFACTORY)
     gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(gtk_ui_manager_get_widget(puim,
                                                                                  "/MainMenu/ViewMenu/DockPanels")),
                                    fDockPanels);
@@ -1216,7 +1218,7 @@ ShowAllPanels(gpointer UNUSED(p), guint UNUSED(n), GtkWidget * UNUSED(pw))
             woPanel[i].showFun();
     }
 
-#if defined(USE_GTKUIMANAGER)
+#if !defined(USE_GTKITEMFACTORY)
     gtk_widget_show((gtk_ui_manager_get_widget(puim, "/MainMenu/ViewMenu/HidePanels")));
     gtk_widget_hide((gtk_ui_manager_get_widget(puim, "/MainMenu/ViewMenu/RestorePanels")));
     gtk_widget_set_sensitive(gtk_ui_manager_get_widget(puim, "/MainMenu/ViewMenu/PanelsMenu/Message"), TRUE);
@@ -1272,7 +1274,7 @@ DoHideAllPanels(int updateEvents)
         }
     }
 
-#if defined(USE_GTKUIMANAGER)
+#if !defined(USE_GTKITEMFACTORY)
     gtk_widget_show((gtk_ui_manager_get_widget(puim, "/MainMenu/ViewMenu/RestorePanels")));
     gtk_widget_set_sensitive((gtk_ui_manager_get_widget(puim, "/MainMenu/ViewMenu/RestorePanels")), TRUE);
     gtk_widget_hide((gtk_ui_manager_get_widget(puim, "/MainMenu/ViewMenu/HidePanels")));
@@ -1314,7 +1316,7 @@ HideAllPanels(gpointer UNUSED(p), guint UNUSED(n), GtkWidget * UNUSED(pw))
     DoHideAllPanels(TRUE);
 }
 
-#if defined(USE_GTKUIMANAGER)
+#if !defined(USE_GTKITEMFACTORY)
 void
 ToggleDockPanels(GtkToggleAction * action, gpointer UNUSED(user_data))
 {
@@ -1720,7 +1722,7 @@ GetPanelWidth(gnubgwindow panel)
 #endif
 
 int
-IsPanelShowVar(gnubgwindow panel, void *p)
+IsPanelShowVar(gnubgwindow panel, void * const p)
 {
     return (p == &woPanel[panel].showing);
 }
