@@ -1,11 +1,11 @@
 /*
- * gtktoolbar.c
+ * Copyright (C) 2003 Joern Thyssen <jth@gnubg.org>
+ * Copyright (C) 2003-2023 the AUTHORS
  *
- * by Joern Thyssen <jth@gnubg.org>, 2003
- *    
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 3 or later of the GNU General Public License as
- * published by the Free Software Foundation.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,10 +13,9 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
- * $Id: gtktoolbar.c,v 1.78 2018/06/26 20:02:21 plm Exp $
+ * $Id: gtktoolbar.c,v 1.88 2023/12/20 14:17:30 plm Exp $
  */
 
 #include "config.h"
@@ -39,11 +38,8 @@
 #include "drawboard.h"
 #include "renderprefs.h"
 #include "gnubgstock.h"
-#if defined(USE_BOARD3D)
-#include "fun3d.h"
-#endif
 
-typedef struct _toolbarwidget {
+typedef struct {
 
     GtkWidget *pwNew;           /* button for "New" */
     GtkWidget *pwOpen;          /* button for "Open" */
@@ -63,12 +59,14 @@ typedef struct _toolbarwidget {
     GtkWidget *pwNextCMarked;   /* button for "Next CMarked" */
     GtkWidget *pwNextMarked;    /* button for "Next CMarked" */
     GtkWidget *pwReset;         /* button for "Reset" */
+    GtkWidget *pwAnalyzeCurrent;        /* button for "Analyze Current" */
+    GtkWidget *pwAnalyzeFile;	/* button for "Analyze File" */
     GtkWidget *pwEdit;          /* button for "Edit" */
     GtkWidget *pwButtonClockwise;       /* button for clockwise */
 
 } toolbarwidget;
 
-#if !defined(USE_GTKUIMANAGER)
+#if defined(USE_GTKITEMFACTORY)
 static void
 ButtonClicked(GtkWidget * UNUSED(pw), char *sz)
 {
@@ -113,7 +111,6 @@ toggle_button_from_images(GtkWidget * pwImageOff, GtkWidget * pwImageOn, char *s
     gtk_container_add(GTK_CONTAINER(pwvbox), gtk_label_new(sz));
     gtk_container_add(GTK_CONTAINER(pw), pwvbox);
 
-
     g_object_set_data_full(G_OBJECT(pw), "toggle_images", aapw, g_free);
 
     return pw;
@@ -121,6 +118,28 @@ toggle_button_from_images(GtkWidget * pwImageOff, GtkWidget * pwImageOn, char *s
 }
 #endif
 
+#if 0	/* not (no longer? not yet?) used */
+extern void
+ToolbarSetAnalyzeCurrent(GtkWidget * pwToolbar, const int f)
+{
+
+    toolbarwidget *ptw = g_object_get_data(G_OBJECT(pwToolbar),
+                                           "toolbarwidget");
+
+    gtk_widget_set_sensitive(ptw->pwAnalyzeCurrent, f);
+}
+
+extern void
+ToolbarSetAnalyzeFile(GtkWidget * pwToolbar, const int f)
+{
+
+    toolbarwidget *ptw = g_object_get_data(G_OBJECT(pwToolbar),
+                                           "toolbarwidget");
+
+    gtk_widget_set_sensitive(ptw->pwAnalyzeFile, f);
+}
+#endif
+
 extern void
 ToolbarSetPlaying(GtkWidget * pwToolbar, const int f)
 {
@@ -132,7 +151,6 @@ ToolbarSetPlaying(GtkWidget * pwToolbar, const int f)
 
 }
 
-
 extern void
 ToolbarSetClockwise(GtkWidget * pwToolbar, const int f)
 {
@@ -140,14 +158,14 @@ ToolbarSetClockwise(GtkWidget * pwToolbar, const int f)
     toolbarwidget *ptw = g_object_get_data(G_OBJECT(pwToolbar),
                                            "toolbarwidget");
 
-#if defined(USE_GTKUIMANAGER)
+#if !defined(USE_GTKITEMFACTORY)
     gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(ptw->pwButtonClockwise), f);
 #else
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ptw->pwButtonClockwise), f);
 #endif
 }
 
-#if defined(USE_GTKUIMANAGER)
+#if !defined(USE_GTKITEMFACTORY)
 extern void
 ToggleClockwise(GtkToggleAction * action, gpointer UNUSED(user_data))
 {
@@ -172,14 +190,8 @@ extern void
 click_swapdirection(void)
 {
     if (!inCallback) {
-#if defined(USE_GTKUIMANAGER)
-        gtk_toggle_action_set_active(GTK_TOGGLE_ACTION(gtk_ui_manager_get_action(puim,
-                                                                                 "/MainMenu/ViewMenu/PlayClockwise")),
-                                     TRUE);
-#else
         toolbarwidget *ptw = g_object_get_data(G_OBJECT(pwToolbar), "toolbarwidget");
         gtk_button_clicked(GTK_BUTTON(ptw->pwButtonClockwise));
-#endif
     }
 }
 
@@ -210,7 +222,7 @@ extern void
 click_edit(void)
 {
     if (!inCallback) {
-#if defined(USE_GTKUIMANAGER)
+#if !defined(USE_GTKITEMFACTORY)
         GtkAction *editstatus = gtk_ui_manager_get_action(puim, "/MainMenu/EditMenu/EditPosition");
         gtk_toggle_action_set_active(GTK_TOGGLE_ACTION(editstatus), !gtk_toggle_action_get_active(GTK_TOGGLE_ACTION(editstatus)));
 #else
@@ -220,7 +232,7 @@ click_edit(void)
     }
 }
 
-#if defined(USE_GTKUIMANAGER)
+#if !defined(USE_GTKITEMFACTORY)
 extern void
 ToggleEdit(GtkToggleAction * action, gpointer UNUSED(user_data))
 {
@@ -283,7 +295,7 @@ ToolbarUpdate(GtkWidget * pwToolbar,
 
     c = C_NONE;
 
-    if (diceShown == DICE_BELOW_BOARD)
+    if (diceShown <= DICE_BELOW_BOARD)
         c = C_ROLLDOUBLE;
 
     if (pms->fDoubled)
@@ -298,13 +310,16 @@ ToolbarUpdate(GtkWidget * pwToolbar,
     if (fEdit || !fPlaying)
         c = C_NONE;
 
-    gtk_widget_set_sensitive(ptw->pwTake, c == C_TAKEDROP || c == C_AGREEDECLINE);
-    gtk_widget_set_sensitive(ptw->pwDrop, c == C_TAKEDROP || c == C_AGREEDECLINE);
-    gtk_widget_set_sensitive(ptw->pwDouble, (c == C_TAKEDROP && !pms->nMatchTo) || c == C_ROLLDOUBLE);
+    /* We want to disable some buttons particularly when we are in the middle
+       of running an analysis in the background => we use !fAnalysisRunning */
 
-    gtk_widget_set_sensitive(ptw->pwSave, plGame != NULL);
-    gtk_widget_set_sensitive(ptw->pwResign, fPlaying && !fEdit);
-    gtk_widget_set_sensitive(ptw->pwHint, fPlaying && !fEdit);
+    gtk_widget_set_sensitive(ptw->pwTake, (c == C_TAKEDROP || c == C_AGREEDECLINE) && !fAnalysisRunning );
+    gtk_widget_set_sensitive(ptw->pwDrop, (c == C_TAKEDROP || c == C_AGREEDECLINE) && !fAnalysisRunning );
+    gtk_widget_set_sensitive(ptw->pwDouble, ((c == C_TAKEDROP && !pms->nMatchTo) || c == C_ROLLDOUBLE) && !fAnalysisRunning );
+
+    gtk_widget_set_sensitive(ptw->pwSave, plGame != NULL && !fAnalysisRunning);
+    gtk_widget_set_sensitive(ptw->pwResign, fPlaying && !fEdit && !fAnalysisRunning);
+    gtk_widget_set_sensitive(ptw->pwHint, fPlaying && !fEdit && !fAnalysisRunning);
     gtk_widget_set_sensitive(ptw->pwPrevMarked, fPlaying && !fEdit);
     gtk_widget_set_sensitive(ptw->pwPrevCMarked, fPlaying && !fEdit);
     gtk_widget_set_sensitive(ptw->pwPrev, fPlaying && !fEdit);
@@ -314,12 +329,15 @@ ToolbarUpdate(GtkWidget * pwToolbar,
     gtk_widget_set_sensitive(ptw->pwNextCMarked, fPlaying && !fEdit);
     gtk_widget_set_sensitive(ptw->pwNextMarked, fPlaying && !fEdit);
     gtk_widget_set_sensitive(ptw->pwEndGame, fPlaying && !fEdit);
-    gtk_widget_set_sensitive(ptw->pwEdit, TRUE);
+    gtk_widget_set_sensitive(ptw->pwEdit, !fAnalysisRunning);
+    gtk_widget_set_sensitive(ptw->pwOpen, !fAnalysisRunning);
+    gtk_widget_set_sensitive(ptw->pwReset, !fAnalysisRunning);
+    gtk_widget_set_sensitive(ptw->pwAnalyzeCurrent, plGame != NULL && !fAnalysisRunning);
 
     return c;
 }
 
-#if !defined(USE_GTKUIMANAGER)
+#if defined(USE_GTKITEMFACTORY)
 static GtkWidget *
 ToolbarAddButton(GtkToolbar * pwToolbar, const char *stockID, const char *label, const char *tooltip,
                  GCallback callback, void *data)
@@ -358,13 +376,13 @@ ToolbarAddSeparator(GtkToolbar * pwToolbar)
 extern GtkWidget *
 ToolbarNew(void)
 {
-#if defined(USE_GTKUIMANAGER)
-    GtkWidget *pwToolbar;
+#if !defined(USE_GTKITEMFACTORY)
+    GtkWidget *pwtb;
     toolbarwidget *ptw = (toolbarwidget *) g_malloc(sizeof(toolbarwidget));
 
-    pwToolbar = gtk_ui_manager_get_widget(puim, "/MainToolBar");
-    g_object_set_data_full(G_OBJECT(pwToolbar), "toolbarwidget", ptw, g_free);
-    gtk_toolbar_set_style(GTK_TOOLBAR(pwToolbar), GTK_TOOLBAR_BOTH);
+    pwtb = gtk_ui_manager_get_widget(puim, "/MainToolBar");
+    g_object_set_data_full(G_OBJECT(pwtb), "toolbarwidget", ptw, g_free);
+    gtk_toolbar_set_style(GTK_TOOLBAR(pwtb), GTK_TOOLBAR_BOTH);
 
     ptw->pwNew = gtk_ui_manager_get_widget(puim, "/MainToolBar/New");
     gtk_tool_item_set_homogeneous(GTK_TOOL_ITEM(ptw->pwNew), TRUE);
@@ -387,6 +405,10 @@ ToolbarNew(void)
     gtk_tool_item_set_homogeneous(GTK_TOOL_ITEM(ptw->pwEndGame), FALSE);
     ptw->pwReset = gtk_ui_manager_get_widget(puim, "/MainToolBar/Undo");
     gtk_tool_item_set_homogeneous(GTK_TOOL_ITEM(ptw->pwReset), TRUE);
+    ptw->pwAnalyzeCurrent = gtk_ui_manager_get_widget(puim, "/MainToolBar/Analyse");
+    gtk_tool_item_set_homogeneous(GTK_TOOL_ITEM(ptw->pwAnalyzeCurrent), TRUE);
+    ptw->pwAnalyzeFile = gtk_ui_manager_get_widget(puim, "/MainToolBar/Analyse File");
+    gtk_tool_item_set_homogeneous(GTK_TOOL_ITEM(ptw->pwAnalyzeFile), TRUE);
     ptw->pwHint = gtk_ui_manager_get_widget(puim, "/MainToolBar/Hint");
     gtk_tool_item_set_homogeneous(GTK_TOOL_ITEM(ptw->pwHint), TRUE);
     ptw->pwEdit = gtk_ui_manager_get_widget(puim, "/MainToolBar/EditPosition");
@@ -420,11 +442,11 @@ ToolbarNew(void)
     gtk_tool_item_set_homogeneous(GTK_TOOL_ITEM(ptw->pwNext), FALSE);
     gtk_tool_button_set_label(GTK_TOOL_BUTTON(ptw->pwNext), "");
 
-    return pwToolbar;
+    return pwtb;
 #else
     GtkWidget *vbox_toolbar;
     GtkToolItem *ti;
-    GtkWidget *pwToolbar, *pwvbox;
+    GtkWidget *pwtb, *pwvbox;
 
     toolbarwidget *ptw;
 
@@ -437,68 +459,68 @@ ToolbarNew(void)
     vbox_toolbar = gtk_vbox_new(FALSE, 0);
 
     g_object_set_data_full(G_OBJECT(vbox_toolbar), "toolbarwidget", ptw, g_free);
-    pwToolbar = gtk_toolbar_new();
-    gtk_toolbar_set_icon_size(GTK_TOOLBAR(pwToolbar), GTK_ICON_SIZE_LARGE_TOOLBAR);
-    gtk_toolbar_set_orientation(GTK_TOOLBAR(pwToolbar), GTK_ORIENTATION_HORIZONTAL);
-    gtk_toolbar_set_style(GTK_TOOLBAR(pwToolbar), GTK_TOOLBAR_BOTH);
-    gtk_box_pack_start(GTK_BOX(vbox_toolbar), pwToolbar, FALSE, FALSE, 0);
+    pwtb = gtk_toolbar_new();
+    gtk_toolbar_set_icon_size(GTK_TOOLBAR(pwtb), GTK_ICON_SIZE_LARGE_TOOLBAR);
+    gtk_orientable_set_orientation(GTK_ORIENTABLE(pwtb), GTK_ORIENTATION_HORIZONTAL);
+    gtk_toolbar_set_style(GTK_TOOLBAR(pwtb), GTK_TOOLBAR_BOTH);
+    gtk_box_pack_start(GTK_BOX(vbox_toolbar), pwtb, FALSE, FALSE, 0);
 
-    gtk_toolbar_set_tooltips(GTK_TOOLBAR(pwToolbar), TRUE);
+    gtk_toolbar_set_tooltips(GTK_TOOLBAR(pwtb), TRUE);
 
     /* New button */
     ptw->pwNew =
-        ToolbarAddButton(GTK_TOOLBAR(pwToolbar), GTK_STOCK_NEW, NULL, _("Start new game, match, session or position"),
+        ToolbarAddButton(GTK_TOOLBAR(pwtb), GTK_STOCK_NEW, NULL, _("Start new game, match, session or position"),
                          G_CALLBACK(GTKNew), NULL);
 
     /* Open button */
     ptw->pwOpen =
-        ToolbarAddButton(GTK_TOOLBAR(pwToolbar), GTK_STOCK_OPEN, NULL, _("Open game, match, session or position"),
+        ToolbarAddButton(GTK_TOOLBAR(pwtb), GTK_STOCK_OPEN, NULL, _("Open game, match, session or position"),
                          G_CALLBACK(GTKOpen), NULL);
 
     /* Save button */
     ptw->pwSave =
-        ToolbarAddButton(GTK_TOOLBAR(pwToolbar), GTK_STOCK_SAVE, NULL, _("Save match, session, game or position"),
+        ToolbarAddButton(GTK_TOOLBAR(pwtb), GTK_STOCK_SAVE, NULL, _("Save match, session, game or position"),
                          G_CALLBACK(GTKSave), NULL);
 
-    ToolbarAddSeparator(GTK_TOOLBAR(pwToolbar));
+    ToolbarAddSeparator(GTK_TOOLBAR(pwtb));
 
     /* Take/accept button */
     ptw->pwTake =
-        ToolbarAddButton(GTK_TOOLBAR(pwToolbar), GNUBG_STOCK_ACCEPT, NULL,
+        ToolbarAddButton(GTK_TOOLBAR(pwtb), GNUBG_STOCK_ACCEPT, NULL,
                          _("Take the offered cube or accept the offered resignation"), G_CALLBACK(ButtonClickedYesNo),
                          "yes");
 
     /* drop button */
     ptw->pwDrop =
-        ToolbarAddButton(GTK_TOOLBAR(pwToolbar), GNUBG_STOCK_REJECT, NULL,
+        ToolbarAddButton(GTK_TOOLBAR(pwtb), GNUBG_STOCK_REJECT, NULL,
                          _("Drop the offered cube or decline the offered resignation"), G_CALLBACK(ButtonClickedYesNo),
                          "no");
 
     /* Double button */
     ptw->pwDouble =
-        ToolbarAddButton(GTK_TOOLBAR(pwToolbar), GNUBG_STOCK_DOUBLE, NULL, _("Double or redouble(beaver)"),
+        ToolbarAddButton(GTK_TOOLBAR(pwtb), GNUBG_STOCK_DOUBLE, NULL, _("Double or redouble(beaver)"),
                          G_CALLBACK(ButtonClicked), "double");
 
     /* Resign button */
     ptw->pwResign =
-        ToolbarAddButton(GTK_TOOLBAR(pwToolbar), GNUBG_STOCK_RESIGN, NULL, _("Resign the current game"),
+        ToolbarAddButton(GTK_TOOLBAR(pwtb), GNUBG_STOCK_RESIGN, NULL, _("Resign the current game"),
                          G_CALLBACK(GTKResign), NULL);
 
     /* End game button */
     ptw->pwEndGame =
-        ToolbarAddButton(GTK_TOOLBAR(pwToolbar), GNUBG_STOCK_END_GAME, NULL, _("Let the computer end the game"),
+        ToolbarAddButton(GTK_TOOLBAR(pwtb), GNUBG_STOCK_END_GAME, NULL, _("Let the computer end the game"),
                          G_CALLBACK(ButtonClicked), "end game");
     gtk_tool_item_set_homogeneous(GTK_TOOL_ITEM(ptw->pwEndGame), FALSE);
 
-    ToolbarAddSeparator(GTK_TOOLBAR(pwToolbar));
+    ToolbarAddSeparator(GTK_TOOLBAR(pwtb));
 
     /* reset button */
     ptw->pwReset =
-        ToolbarAddButton(GTK_TOOLBAR(pwToolbar), GTK_STOCK_UNDO, NULL, _("Undo moves"), G_CALLBACK(GTKUndo), NULL);
+        ToolbarAddButton(GTK_TOOLBAR(pwtb), GTK_STOCK_UNDO, NULL, _("Undo moves"), G_CALLBACK(GTKUndo), NULL);
 
     /* Hint button */
     ptw->pwHint =
-        ToolbarAddButton(GTK_TOOLBAR(pwToolbar), GNUBG_STOCK_HINT, NULL, _("Show the best moves or cube action"),
+        ToolbarAddButton(GTK_TOOLBAR(pwtb), GNUBG_STOCK_HINT, NULL, _("Show the best moves or cube action"),
                          G_CALLBACK(ButtonClicked), "hint");
 
     /* edit button */
@@ -508,7 +530,7 @@ ToolbarNew(void)
     gtk_container_add(GTK_CONTAINER(pwvbox), gtk_label_new(_("Edit")));
     gtk_container_add(GTK_CONTAINER(ptw->pwEdit), pwvbox);
     g_signal_connect(G_OBJECT(ptw->pwEdit), "toggled", G_CALLBACK(ToolbarToggleEdit), NULL);
-    ti = GTK_TOOL_ITEM(ToolbarAddWidget(GTK_TOOLBAR(pwToolbar), ptw->pwEdit, _("Toggle Edit Mode")));
+    ti = GTK_TOOL_ITEM(ToolbarAddWidget(GTK_TOOLBAR(pwtb), ptw->pwEdit, _("Toggle Edit Mode")));
     gtk_tool_item_set_homogeneous(ti, TRUE);
 
     /* direction of play */
@@ -518,43 +540,58 @@ ToolbarNew(void)
                                   _("Direction"));
     g_signal_connect(G_OBJECT(ptw->pwButtonClockwise), "toggled", G_CALLBACK(ToolbarToggleClockwise), ptw);
 
-    ToolbarAddWidget(GTK_TOOLBAR(pwToolbar), ptw->pwButtonClockwise, _("Reverse direction of play"));
+    ToolbarAddWidget(GTK_TOOLBAR(pwtb), ptw->pwButtonClockwise, _("Reverse direction of play"));
+
+    ToolbarAddSeparator(GTK_TOOLBAR(pwtb));
+    
+    /* Analyze current match button */
+
+    ptw->pwAnalyzeCurrent =
+        ToolbarAddButton(GTK_TOOLBAR(pwtb), GTK_STOCK_EXECUTE, _("Analyse"),
+        _("Analyse current match (set default behaviour in Settings -> Analysis)"),
+        G_CALLBACK(GTKAnalyzeCurrent), NULL);
+
+    /* Analyze file button */
+    ptw->pwAnalyzeFile =
+        ToolbarAddButton(GTK_TOOLBAR(pwtb), GTK_STOCK_DIRECTORY, _("Analyse File"),
+        _("Analyze match from file (set default behaviour in Settings -> Analysis -> Analysis Buttons)"),
+        G_CALLBACK(GTKAnalyzeFile), NULL);
 
     ti = gtk_separator_tool_item_new();
     gtk_tool_item_set_expand(GTK_TOOL_ITEM(ti), TRUE);
     gtk_separator_tool_item_set_draw(GTK_SEPARATOR_TOOL_ITEM(ti), FALSE);
-    gtk_toolbar_insert(GTK_TOOLBAR(pwToolbar), ti, -1);
+    gtk_toolbar_insert(GTK_TOOLBAR(pwtb), ti, -1);
 
     ptw->pwPrevCMarked =
-        ToolbarAddButton(GTK_TOOLBAR(pwToolbar), GNUBG_STOCK_GO_PREV_MARKED, "", _("Go to Previous Marked"),
+        ToolbarAddButton(GTK_TOOLBAR(pwtb), GNUBG_STOCK_GO_PREV_MARKED, "", _("Go to Previous Marked"),
                          G_CALLBACK(ButtonClicked), "previous marked");
     gtk_tool_item_set_homogeneous(GTK_TOOL_ITEM(ptw->pwPrevCMarked), FALSE);
     ptw->pwPrevMarked =
-        ToolbarAddButton(GTK_TOOLBAR(pwToolbar), GNUBG_STOCK_GO_PREV_CMARKED, "", _("Go to Previous CMarked"),
+        ToolbarAddButton(GTK_TOOLBAR(pwtb), GNUBG_STOCK_GO_PREV_CMARKED, "", _("Go to Previous CMarked"),
                          G_CALLBACK(ButtonClicked), "previous cmarked");
     gtk_tool_item_set_homogeneous(GTK_TOOL_ITEM(ptw->pwPrevMarked), FALSE);
     ptw->pwPrev =
-        ToolbarAddButton(GTK_TOOLBAR(pwToolbar), GNUBG_STOCK_GO_PREV, "", _("Go to Previous Roll"),
+        ToolbarAddButton(GTK_TOOLBAR(pwtb), GNUBG_STOCK_GO_PREV, "", _("Go to Previous Roll"),
                          G_CALLBACK(ButtonClicked), "previous roll");
     gtk_tool_item_set_homogeneous(GTK_TOOL_ITEM(ptw->pwPrev), FALSE);
     ptw->pwPrevGame =
-        ToolbarAddButton(GTK_TOOLBAR(pwToolbar), GNUBG_STOCK_GO_PREV_GAME, "", _("Go to Previous Game"),
+        ToolbarAddButton(GTK_TOOLBAR(pwtb), GNUBG_STOCK_GO_PREV_GAME, "", _("Go to Previous Game"),
                          G_CALLBACK(ButtonClicked), "previous game");
     gtk_tool_item_set_homogeneous(GTK_TOOL_ITEM(ptw->pwPrevGame), FALSE);
     ptw->pwNextGame =
-        ToolbarAddButton(GTK_TOOLBAR(pwToolbar), GNUBG_STOCK_GO_NEXT_GAME, "", _("Go to Next Game"),
+        ToolbarAddButton(GTK_TOOLBAR(pwtb), GNUBG_STOCK_GO_NEXT_GAME, "", _("Go to Next Game"),
                          G_CALLBACK(ButtonClicked), "next game");
     gtk_tool_item_set_homogeneous(GTK_TOOL_ITEM(ptw->pwNextGame), FALSE);
     ptw->pwNext =
-        ToolbarAddButton(GTK_TOOLBAR(pwToolbar), GNUBG_STOCK_GO_NEXT, "", _("Go to Next Roll"),
+        ToolbarAddButton(GTK_TOOLBAR(pwtb), GNUBG_STOCK_GO_NEXT, "", _("Go to Next Roll"),
                          G_CALLBACK(ButtonClicked), "next roll");
     gtk_tool_item_set_homogeneous(GTK_TOOL_ITEM(ptw->pwNext), FALSE);
     ptw->pwNextCMarked =
-        ToolbarAddButton(GTK_TOOLBAR(pwToolbar), GNUBG_STOCK_GO_NEXT_CMARKED, "", _("Go to Next CMarked"),
+        ToolbarAddButton(GTK_TOOLBAR(pwtb), GNUBG_STOCK_GO_NEXT_CMARKED, "", _("Go to Next CMarked"),
                          G_CALLBACK(ButtonClicked), "next cmarked");
     gtk_tool_item_set_homogeneous(GTK_TOOL_ITEM(ptw->pwNextCMarked), FALSE);
     ptw->pwNextMarked =
-        ToolbarAddButton(GTK_TOOLBAR(pwToolbar), GNUBG_STOCK_GO_NEXT_MARKED, "", _("Go to Next Marked"),
+        ToolbarAddButton(GTK_TOOLBAR(pwtb), GNUBG_STOCK_GO_NEXT_MARKED, "", _("Go to Next Marked"),
                          G_CALLBACK(ButtonClicked), "next marked");
     gtk_tool_item_set_homogeneous(GTK_TOOL_ITEM(ptw->pwNextMarked), FALSE);
 
@@ -562,7 +599,7 @@ ToolbarNew(void)
 #endif
 }
 
-#if !defined(USE_GTKUIMANAGER)
+#if defined(USE_GTKITEMFACTORY)
 static GtkWidget *
 firstChild(GtkWidget * widget)
 {
@@ -606,7 +643,7 @@ SetToolbarItemStyle(gpointer data, gpointer user_data)
 extern void
 SetToolbarStyle(int value)
 {
-#if defined(USE_GTKUIMANAGER)
+#if !defined(USE_GTKITEMFACTORY)
     toolbarwidget *ptw = g_object_get_data(G_OBJECT(pwToolbar), "toolbarwidget");
     GtkWidget *img = gtk_image_new_from_stock(fClockwise ? GNUBG_STOCK_CLOCKWISE : GNUBG_STOCK_ANTI_CLOCKWISE,
                                               GTK_ICON_SIZE_SMALL_TOOLBAR);
@@ -615,7 +652,7 @@ SetToolbarStyle(int value)
 #endif
 
     if (value != nToolbarStyle) {
-#if defined(USE_GTKUIMANAGER)
+#if !defined(USE_GTKITEMFACTORY)
         gtk_toolbar_set_style(GTK_TOOLBAR(pwToolbar), (GtkToolbarStyle) value);
 #else
         toolbarwidget *ptw = g_object_get_data(G_OBJECT(pwToolbar), "toolbarwidget");
@@ -628,10 +665,11 @@ SetToolbarStyle(int value)
         /* Resize handle box parent */
         gtk_widget_queue_resize(pwToolbar);
         nToolbarStyle = value;
-#if !defined(USE_GTKUIMANAGER)
+#if defined(USE_GTKITEMFACTORY)
         gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM
                                        (gtk_item_factory_get_widget_by_action(pif, value + TOOLBAR_ACTION_OFFSET)),
                                        TRUE);
 #endif
+        UserCommand("save settings");
     }
 }
