@@ -1,11 +1,11 @@
 /*
- * relational.c
+ * Copyright (C) 2004 Joern Thyssen <jth@gnubg.org>
+ * Copyright (C) 2004-2023 the AUTHORS
  *
- * by Joern Thyssen <jth@gnubg.org>, 2004.
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 3 or later of the GNU General Public License as
- * published by the Free Software Foundation.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,10 +13,9 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
- * $Id: relational.c,v 1.77 2017/09/30 22:03:34 plm Exp $
+ * $Id: relational.c,v 1.87 2023/12/06 22:01:46 plm Exp $
  */
 
 #include "config.h"
@@ -38,7 +37,6 @@
 #include "util.h"
 #include <glib/gstdio.h>
 #include <glib.h>
-#include "glib-ext.h"
 
 static int
 RelationalMatchExists(DBProvider * pdb)
@@ -52,15 +50,18 @@ RelationalMatchExists(DBProvider * pdb)
 static int
 GameOver(void)
 {
-    int anFinalScore[2];
-    int nMatch;
     const listOLD *firstGame = lMatch.plNext->p;
+
     if (firstGame) {
         const moverecord *pmr = firstGame->plNext->p;
         if (pmr) {
+            int nMatch;
+
             g_assert(pmr->mt == MOVE_GAMEINFO);
             nMatch = pmr->g.nMatch;
             if (ms.nMatchTo) {  /* Match - check someone has won */
+                int anFinalScore[2];
+
                 return (getFinalScore(anFinalScore) && ((anFinalScore[0] >= nMatch)
                                                         || (anFinalScore[1] >= nMatch)));
             } else {            /* Session - check game over */
@@ -127,23 +128,14 @@ MatchResult(int nMatchTo)
     int result = 0;
     int anFinalScore[2];
     if (nMatchTo && getFinalScore(anFinalScore)) {
-        if (anFinalScore[0] > nMatchTo)
+        if (anFinalScore[0] >= nMatchTo)
             result = -1;
-        else if (anFinalScore[1] > nMatchTo)
+        else if (anFinalScore[1] >= nMatchTo)
             result = 1;
     }
     return result;
 }
 
-float
-Ratio(float a, int b)
-{
-    if (b != 0)
-        return a / (float) b;
-    else
-        return 0;
-}
-
 #define NS(x) (x == NULL) ? "NULL" : x
 #define APPENDF(x,y) {g_string_append_printf(column, "%s, ", x); \
 	g_string_append_printf(value, "'%s', ", g_ascii_dtostr(tmpf, G_ASCII_DTOSTR_BUF_SIZE, y));}
@@ -197,9 +189,9 @@ AddStats(DBProvider * pdb, int gm_id, int player_id, int player, const char *tab
     APPENDI("very_bad_moves", sc->anMoves[player][SKILL_VERYBAD]);
     APPENDF("chequer_error_total_normalised", sc->arErrorCheckerplay[player][0]);
     APPENDF("chequer_error_total", sc->arErrorCheckerplay[player][1]);
-    APPENDF("chequer_error_per_move_normalised", Ratio(sc->arErrorCheckerplay[player][0], unforced));
-    APPENDF("chequer_error_per_move", Ratio(sc->arErrorCheckerplay[player][1], unforced));
-    APPENDU("chequer_rating", GetRating(Ratio(scMatch.arErrorCheckerplay[player][0], unforced)));
+    APPENDF("chequer_error_per_move_normalised", Ratiof(sc->arErrorCheckerplay[player][0], unforced));
+    APPENDF("chequer_error_per_move", Ratiof(sc->arErrorCheckerplay[player][1], unforced));
+    APPENDU("chequer_rating", GetRating(Ratiof(scMatch.arErrorCheckerplay[player][0], unforced)));
     APPENDI("very_lucky_rolls", sc->anLuck[player][LUCK_VERYGOOD]);
     APPENDI("lucky_rolls", sc->anLuck[player][LUCK_GOOD]);
     APPENDI("unmarked_rolls", sc->anLuck[player][LUCK_NONE]);
@@ -207,9 +199,9 @@ AddStats(DBProvider * pdb, int gm_id, int player_id, int player, const char *tab
     APPENDI("very_unlucky_rolls", sc->anLuck[player][LUCK_VERYBAD]);
     APPENDF("luck_total_normalised", sc->arLuck[player][0]);
     APPENDF("luck_total", sc->arLuck[player][1]);
-    APPENDF("luck_per_move_normalised", Ratio(sc->arLuck[player][0], totalmoves));
-    APPENDF("luck_per_move", Ratio(sc->arLuck[player][1], totalmoves));
-    APPENDU("luck_rating", getLuckRating(Ratio(sc->arLuck[player][0], totalmoves)));
+    APPENDF("luck_per_move_normalised", Ratiof(sc->arLuck[player][0], totalmoves));
+    APPENDF("luck_per_move", Ratiof(sc->arLuck[player][1], totalmoves));
+    APPENDU("luck_rating", getLuckRating(Ratiof(sc->arLuck[player][0], totalmoves)));
     APPENDI("total_cube_decisions", sc->anTotalCube[player]);
     APPENDI("close_cube_decisions", sc->anCloseCube[player]);
     APPENDI("doubles", sc->anDouble[player]);
@@ -233,28 +225,28 @@ AddStats(DBProvider * pdb, int gm_id, int player_id, int player, const char *tab
     APPENDF("error_wrong_doubles_above_tg", sc->arErrorWrongDoubleTG[player][1]);
     APPENDF("error_wrong_takes", sc->arErrorWrongTake[player][1]);
     APPENDF("error_wrong_passes", sc->arErrorWrongPass[player][1]);
-    APPENDF("cube_error_total_normalised", errorskill * sc->anCloseCube[player]);
-    APPENDF("cube_error_total", errorcost * sc->anCloseCube[player]);
+    APPENDF("cube_error_total_normalised", errorskill * (float) sc->anCloseCube[player]);
+    APPENDF("cube_error_total", errorcost * (float) sc->anCloseCube[player]);
     APPENDF("cube_error_per_move_normalised", errorskill);
     APPENDF("cube_error_per_move", errorcost);
-    APPENDU("cube_rating", GetRating(errorskill));;
-    APPENDF("overall_error_total_normalised", errorskill * sc->anCloseCube[player] + sc->arErrorCheckerplay[player][0]);
-    APPENDF("overall_error_total", errorcost * sc->anCloseCube[player] + sc->arErrorCheckerplay[player][1]);
+    APPENDU("cube_rating", GetRating(errorskill));
+    APPENDF("overall_error_total_normalised", errorskill * (float) sc->anCloseCube[player] + sc->arErrorCheckerplay[player][0]);
+    APPENDF("overall_error_total", errorcost * (float) sc->anCloseCube[player] + sc->arErrorCheckerplay[player][1]);
     APPENDF("overall_error_per_move_normalised",
-            Ratio(errorskill * sc->anCloseCube[player] +
+            Ratiof(errorskill * (float) sc->anCloseCube[player] +
                   sc->arErrorCheckerplay[player][0], sc->anCloseCube[player] + unforced));
     APPENDF("overall_error_per_move",
-            Ratio(errorcost * sc->anCloseCube[player] +
+            Ratiof(errorcost * (float) sc->anCloseCube[player] +
                   sc->arErrorCheckerplay[player][1], sc->anCloseCube[player] + unforced));
     APPENDU("overall_rating",
-            GetRating(Ratio
-                      (errorskill * sc->anCloseCube[player] +
+            GetRating(Ratiof
+                      (errorskill * (float) sc->anCloseCube[player] +
                        sc->arErrorCheckerplay[player][0], sc->anCloseCube[player] + unforced)));
     APPENDF("actual_result", sc->arActualResult[player]);
     APPENDF("luck_adjusted_result", sc->arLuckAdj[player]);
     APPENDI("snowie_moves", totalmoves + scMatch.anTotalMoves[!player]);
     APPENDF("snowie_error_rate_per_move",
-            Ratio(errorskill * scMatch.anCloseCube[player] +
+            Ratiof(errorskill * (float) scMatch.anCloseCube[player] +
                   scMatch.arErrorCheckerplay[player][0], totalmoves + scMatch.anTotalMoves[!player]));
     /*time */
     APPENDI("time_penalties", 0);
@@ -281,10 +273,10 @@ AddStats(DBProvider * pdb, int gm_id, int player_id, int player, const char *tab
     /* for money sessions only */
     if (scMatch.fDice && !nMatchTo && scMatch.nGames > 1) {
 
-        APPENDF("actual_advantage", scMatch.arActualResult[player] / scMatch.nGames);
-        APPENDF("actual_advantage_ci", 1.95996f * sqrtf(scMatch.arVarianceActual[player] / scMatch.nGames));
-        APPENDF("luck_adjusted_advantage", scMatch.arLuckAdj[player] / scMatch.nGames);
-        APPENDF("luck_adjusted_advantage_ci", 1.95996f * sqrtf(scMatch.arVarianceLuckAdj[player] / scMatch.nGames));
+        APPENDF("actual_advantage", scMatch.arActualResult[player] / (float) scMatch.nGames);
+        APPENDF("actual_advantage_ci", 1.95996f * sqrtf(scMatch.arVarianceActual[player] / (float) scMatch.nGames));
+        APPENDF("luck_adjusted_advantage", scMatch.arLuckAdj[player] / (float) scMatch.nGames);
+        APPENDF("luck_adjusted_advantage_ci", 1.95996f * sqrtf(scMatch.arVarianceLuckAdj[player] / (float) scMatch.nGames));
     }
 
     g_string_truncate(column, column->len - 2);
@@ -305,7 +297,7 @@ CreateDatabase(DBProvider * pdb)
     char line[1024];
 
     gchar *szFile = BuildFilename("gnubg.sql");
-    FILE *fp = gnubg_g_fopen(szFile, "r");
+    FILE *fp = g_fopen(szFile, "r");
 
     if (!fp) {
         g_free(szFile);
@@ -378,17 +370,32 @@ static void
 AddGames(DBProvider * pdb, int session_id, int player_id0, int player_id1)
 {
     int gamenum = 0;
-    listOLD *plGame, *pl = lMatch.plNext;
-    while ((plGame = pl->p) != NULL) {
+    listOLD *plg, *pl = lMatch.plNext;
+    while ((plg = pl->p) != NULL) {
         int game_id = GetNextId(pdb, "game");
-        moverecord *pmr = plGame->plNext->p;
+        int result = 0;
+        moverecord *pmr = plg->plNext->p;
         xmovegameinfo *pmgi = &pmr->g;
+        char *buf;
 
-        char *buf = g_strdup_printf("INSERT INTO game(game_id, session_id, player_id0, player_id1, "
+        switch(pmgi->fWinner) {
+            case 0:
+                result = pmgi->nPoints;
+                break;
+            case 1:
+                result = -pmgi->nPoints;
+                break;
+            case -1:
+                break;
+            default:
+                g_assert_not_reached();
+        }
+
+        buf = g_strdup_printf("INSERT INTO game(game_id, session_id, player_id0, player_id1, "
                                     "score_0, score_1, result, added, game_number, crawford) "
                                     "VALUES (%d, %d, %d, %d, %d, %d, %d, CURRENT_TIMESTAMP, %d, %d )",
                                     game_id, session_id, player_id0, player_id1,
-                                    pmgi->anScore[0], pmgi->anScore[1], pmgi->nPoints, ++gamenum, pmr->g.fCrawfordGame);
+                                    pmgi->anScore[0], pmgi->anScore[1], result, ++gamenum, pmr->g.fCrawfordGame);
 
         if (pdb->UpdateCommand(buf)) {
             AddStats(pdb, game_id, player_id0, 0, "gamestat", ms.nMatchTo, &(pmgi->sc));
@@ -403,7 +410,7 @@ extern void
 CommandRelationalAddMatch(char *sz)
 {
     DBProvider *pdb;
-    char *buf, *buf2, *date;
+    char *buf, *date;
     char warnings[1024] = "";
     int session_id, existing_id, player_id0, player_id1;
     char *arg = NULL;
@@ -436,7 +443,9 @@ CommandRelationalAddMatch(char *sz)
     }
     existing_id = RelationalMatchExists(pdb);
     if (existing_id != -1) {
-        if (!quiet && !GetInputYN(_("Match exists, overwrite?")))
+        char *buf2;
+
+        if (!quiet && !GetInputYN(_("Match exists in database, overwrite?")))
             return;
 
         /* Remove any game stats and games */
@@ -533,8 +542,6 @@ relational_player_stats_get(const char *player0, const char *player1)
     DBProvider *pdb = NULL;
     char *query[2];
     int i;
-    char *buf;
-    RowSet *rs;
     statcontext *psc;
 
     g_return_val_if_fail(player0, NULL);
@@ -545,8 +552,10 @@ relational_player_stats_get(const char *player0, const char *player1)
     id0 = GetPlayerId(pdb, player0);
     if (player1)
         id1 = GetPlayerId(pdb, player1);
-    if (id0 == -1 || (player1 && id1 == -1))
+    if (id0 == -1 || (player1 && id1 == -1)) {
+        pdb->Disconnect();
         return NULL;
+    }
 
     psc = g_new0(statcontext, 1);
 
@@ -572,7 +581,7 @@ relational_player_stats_get(const char *player0, const char *player1)
 
     IniStatcontext(psc);
     for (i = 0; i < 2; ++i) {
-        buf = g_strdup_printf("SUM(total_moves),"
+        char *buf = g_strdup_printf("SUM(total_moves),"
                               "SUM(unforced_moves),"
                               "SUM(total_cube_decisions),"
                               "SUM(close_cube_decisions),"
@@ -602,10 +611,14 @@ relational_player_stats_get(const char *player0, const char *player1)
                               "SUM(error_wrong_takes_normalised),"
                               "SUM(error_wrong_passes_normalised),"
                               "SUM(luck_total_normalised)" "from matchstat " "%s", query[i]);
-        rs = pdb->Select(buf);
+        RowSet *rs = pdb->Select(buf);
         g_free(buf);
-        if ((!rs) || !strtol(rs->data[1][0], NULL, 0))
+        g_free(query[i]);
+
+        if ((!rs) || !strtol(rs->data[1][0], NULL, 0)) {
+            pdb->Disconnect();
             return NULL;
+        }
         psc->anTotalMoves[i] = (int) strtol(rs->data[1][0], NULL, 0);
         psc->anUnforcedMoves[i] = (int) strtol(rs->data[1][1], NULL, 0);
         psc->anTotalCube[i] = (int) strtol(rs->data[1][2], NULL, 0);
@@ -642,6 +655,8 @@ relational_player_stats_get(const char *player0, const char *player1)
     psc->fCube = 1;
     psc->fDice = 1;
 
+    pdb->Disconnect();
+
     return psc;
 }
 
@@ -649,7 +664,7 @@ relational_player_stats_get(const char *player0, const char *player1)
 extern void
 CommandRelationalShowDetails(char *sz)
 {
-    gchar output[4096];
+    gchar buffer[STATCONTEXT_MAXSIZE];
     statcontext *psc;
 
     gchar *player0 = NextToken(&sz);
@@ -668,16 +683,16 @@ CommandRelationalShowDetails(char *sz)
         return;
     }
 
-    DumpStatcontext(output, psc, player0, player1 ? player1 : _("Opponents"), 0);
+    DumpStatcontext(buffer, psc, player0, player1 ? player1 : _("Opponents"), 0);
     g_free(psc);
 #if USE_GTK
     if (fX) {
-        GTKTextWindow(output, _("Player statistics"), DT_INFO, NULL);
+        GTKTextWindow(buffer, _("Player statistics"), DT_INFO, NULL);
     } else
 #endif
     {
         outputl(_("Player statistics\n\n"));
-        outputl(output);
+        outputl(buffer);
     }
 }
 
@@ -763,7 +778,6 @@ CommandRelationalEraseAll(char *UNUSED(sz))
 extern void
 CommandRelationalSelect(char *sz)
 {
-    unsigned int i, j;
     RowSet *rs;
 
     if (!sz || !*sz) {
@@ -785,7 +799,7 @@ CommandRelationalSelect(char *sz)
         GtkShowQuery(rs);
     else
 #endif
-        for (i = 0; i < rs->rows; i++) {
+        for (unsigned int i = 0; i < rs->rows; i++) {
             if (i == 1) {       /* Underline headings */
                 char *line, *p;
                 unsigned int k;
@@ -795,7 +809,7 @@ CommandRelationalSelect(char *sz)
                     if (k != 0)
                         totalwidth += 2;
                 }
-                line = malloc(totalwidth + 1);
+                line = g_malloc(totalwidth + 1);
                 memset(line, '-', totalwidth);
                 p = line;
                 for (k = 0; k < rs->cols - 1; k++) {
@@ -805,10 +819,10 @@ CommandRelationalSelect(char *sz)
                 }
                 line[totalwidth] = '\0';
                 outputl(line);
-                free(line);
+                g_free(line);
             }
 
-            for (j = 0; j < rs->cols; j++) {
+            for (unsigned int j = 0; j < rs->cols; j++) {
                 if (j > 0)
                     output(" | ");
 
