@@ -1,11 +1,11 @@
 /*
- * relational.h
+ * Copyright (C) 2004 Joern Thyssen <jth@gnubg.org>
+ * Copyright (C) 2004-2023 the AUTHORS
  *
- * by Joern Thyssen <jth@gnubg.org>, 2004.
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 3 or later of the GNU General Public License as
- * published by the Free Software Foundation.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,10 +13,9 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
- * $Id: relational.h,v 1.16 2018/06/16 14:50:07 plm Exp $
+ * $Id: relational.h,v 1.18 2023/11/20 21:00:06 plm Exp $
  */
 
 #ifndef RELATIONAL_H
@@ -29,9 +28,17 @@
 
 #define DB_VERSION 1
 
-
 extern int RelationalUpdatePlayerDetails(const char *oldName, const char *newName, const char *newNotes);
-extern float Ratio(float a, int b);
 extern statcontext *relational_player_stats_get(const char *player0, const char *player1);
 
+static inline float Ratiof(float a, int b)
+{
+    return b ? a / (float) b : 0.0f;
+}
+
+static inline double Ratio(double a, int b)
+{
+    return b ? a / (double) b : 0.0f;
+}
+
 #endif                          /* RELATIONAL_H */
