@@ -1,11 +1,11 @@
 /*
- * rollout.c
+ * Copyright (C) 1999-2003 Gary Wong <gtw@gnu.org>
+ * Copyright (C) 2000-2021 the AUTHORS
  *
- * by Gary Wong <gary@cs.arizona.edu>, 1999, 2000, 2001.
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 3 or later of the GNU General Public License as
- * published by the Free Software Foundation.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,10 +13,9 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
- * $Id: rollout.c,v 1.243 2014/08/09 16:07:40 mdpetch Exp $
+ * $Id: rollout.c,v 1.263 2023/12/14 07:02:38 plm Exp $
  */
 
 #include "config.h"
@@ -32,7 +31,7 @@
 #include <time.h>
 
 #include "backgammon.h"
-#if USE_GTK
+#if defined(USE_GTK)
 #include "gtkgame.h"
 #endif
 #include "matchid.h"
@@ -40,8 +39,11 @@
 #include "format.h"
 #include "multithread.h"
 #include "rollout.h"
+#include "lib/simd.h"
 
-#if !LOCKING_VERSION
+#define LogCubeClamped(n) (n < (1 << STAT_MAXCUBE) ? LogCube(n) : (STAT_MAXCUBE - 1))
+
+#if !defined(LOCKING_VERSION)
 
 f_BasicCubefulRollout BasicCubefulRollout = BasicCubefulRolloutNoLocking;
 #define BasicCubefulRollout BasicCubefulRolloutNoLocking
@@ -114,7 +116,7 @@ board_to_sgf(FILE * logfp, const unsigned int anBoard[25], int direction)
         fprintf(logfp, "[y]");
 }
 
-extern FILE *
+static FILE *
 log_game_start(const char *name, const cubeinfo * pci, int fCubeful, TanBoard anBoard)
 {
     time_t t = time(0);
@@ -167,7 +169,7 @@ log_game_start(const char *name, const cubeinfo * pci, int fCubeful, TanBoard an
     return logfp;
 }
 
-extern void
+static void
 log_game_over(FILE * logfp)
 {
     if (!logfp)
@@ -176,7 +178,7 @@ log_game_over(FILE * logfp)
     fclose(logfp);
 }
 
-extern void
+static void
 QuasiRandomSeed(perArray * pArray, int n)
 {
 
@@ -193,7 +195,7 @@ QuasiRandomSeed(perArray * pArray, int n)
     irandinit(&rc, TRUE);
 
     for (i = 0; i < 6; i++)
-        for (j = i /* no need for permutations below the diagonal */ ; j < 128; j++) {
+        for (j = i /* no need for permutations below the diagonal */ ; j < QRLEN; j++) {
             for (k = 0; k < 36; k++)
                 pArray->aaanPermutation[i][j][k] = k;
             for (k = 0; k < 35; k++) {
@@ -218,13 +220,12 @@ RolloutDice(int iTurn, int iGame,
     if (fInitial && !iTurn) {
         /* rollout of initial position: no doubles allowed */
         if (fRotate) {
-            unsigned int j;
 
             if (!iGame)
-                nSkip = 0;
+                MT_SafeSet(&nSkip, 0);
 
-            for (;; nSkip++) {
-                j = dicePerms->aaanPermutation[0][0][(iGame + nSkip) % 36];
+            for (;; MT_SafeInc(&nSkip)) {
+                unsigned int j = dicePerms->aaanPermutation[0][0][(iGame + MT_SafeGet(&nSkip)) % 36];
 
                 anDice[0] = j / 6 + 1;
                 anDice[1] = j % 6 + 1;
@@ -239,17 +240,17 @@ RolloutDice(int iTurn, int iGame,
                 int n;
                 if ((n = RollDice(anDice, rngx, rngctx)) != 0)
                     return n;
-            } while (fInitial && !iTurn && anDice[0] == anDice[1]);
+            } while (anDice[0] == anDice[1]);
 
             return 0;
         }
-    } else if (fRotate && iTurn < 128) {
+    } else if (fRotate && iTurn < QRLEN) {
         unsigned int i,         /* the "generation" of the permutation */
          j,                     /* the number we're permuting */
          k;                     /* 36**i */
 
         for (i = 0, j = 0, k = 1; i < 6 && i <= (unsigned int) iTurn; i++, k *= 36)
-            j = dicePerms->aaanPermutation[i][iTurn][((iGame + nSkip) / k + j) % 36];
+            j = dicePerms->aaanPermutation[i][iTurn][((iGame + MT_SafeGet(&nSkip)) / k + j) % 36];
 
         anDice[0] = j / 6 + 1;
         anDice[1] = j % 6 + 1;
@@ -263,11 +264,11 @@ extern void
 ClosedBoard(int afClosedBoard[2], const TanBoard anBoard)
 {
 
-    int i, j, n;
+    int i, j;
 
     for (i = 0; i < 2; i++) {
+        int n = 0;
 
-        n = 0;
         for (j = 0; j < 6; j++) {
             if (anBoard[i][j] > 1)
                 n++;
@@ -287,7 +288,9 @@ static volatile unsigned int initial_game_count;
 
 #endif
 
-extern void initRolloutstat(rolloutstat * prs);
+#if !defined(LOCKING_VERSION)
+static void initRolloutstat(rolloutstat * prs);
+#endif
 
 /* called with 
  * cube decision                  move rollout
@@ -347,6 +350,8 @@ BasicCubefulRollout(unsigned int aanBoard[][2][25],
 
     int nLateEvals = prc->fLateEvals ? prc->nLate : 0x7fffffff;
 
+    int useVarRedn = prc->fVarRedn;
+
     /* Make local copy of cubeinfo struct, since it
      * may be modified */
     cubeinfo *pciLocal = g_alloca(cci * sizeof(cubeinfo));
@@ -360,7 +365,12 @@ BasicCubefulRollout(unsigned int aanBoard[][2][25],
     float arMean[NUM_ROLLOUT_OUTPUTS];
     unsigned int aaanBoard[6][6][2][25];
     int aanMoves[6][6][8];
+#if defined(USE_SIMD_INSTRUCTIONS)
+#define NUM_ROLLOUT_OUTPUTS_PADDED (NUM_ROLLOUT_OUTPUTS + VEC_SIZE - (NUM_ROLLOUT_OUTPUTS % VEC_SIZE))
+    SSE_ALIGN(float aaar[6][6][NUM_ROLLOUT_OUTPUTS_PADDED]);
+#else
     float aaar[6][6][NUM_ROLLOUT_OUTPUTS];
+#endif
 
     evalcontext ecCubeless0ply = { FALSE, 0, FALSE, TRUE, 0.0 };
     evalcontext ecCubeful0ply = { TRUE, 0, FALSE, TRUE, 0.0 };
@@ -368,7 +378,7 @@ BasicCubefulRollout(unsigned int aanBoard[][2][25],
     /* local pointers to the eval contexts to use */
     evalcontext *pecCube[2], *pecChequer[2];
 
-    if (prc->fVarRedn) {
+    if (useVarRedn) {
 
         /*
          * Create evaluation context one ply deep
@@ -469,7 +479,7 @@ BasicCubefulRollout(unsigned int aanBoard[][2][25],
 
                         /* update statistics */
                         if (aarsStatistics)
-                            MT_SafeInc(&aarsStatistics[ici][pci->fMove].acDoubleTake[LogCube(pci->nCube)]);
+                            MT_SafeInc(&aarsStatistics[ici][pci->fMove].acDoubleTake[LogCubeClamped(pci->nCube)]);
 
                         SetCubeInfo(pci, 2 * pci->nCube, !pci->fMove, pci->fMove, pci->nMatchTo,
                                     pci->anScore, pci->fCrawford, pci->fJacoby, pci->fBeavers, pci->bgv);
@@ -507,9 +517,9 @@ BasicCubefulRollout(unsigned int aanBoard[][2][25],
                         /* update statistics */
 
                         if (aarsStatistics) {
-                            MT_SafeInc(&aarsStatistics[ici][pci->fMove].acDoubleDrop[LogCube(pci->nCube)]);
-                            MT_SafeInc(&aarsStatistics[ici][pci->fMove].acWin[LogCube(pci->nCube)]);
-                        };
+                            MT_SafeInc(&aarsStatistics[ici][pci->fMove].acDoubleDrop[LogCubeClamped(pci->nCube)]);
+                            MT_SafeInc(&aarsStatistics[ici][pci->fMove].acWin[LogCubeClamped(pci->nCube)]);
+                        }
 
                         break;
 
@@ -568,7 +578,7 @@ BasicCubefulRollout(unsigned int aanBoard[][2][25],
 
                 /* Find best move :-) */
 
-                if (prc->fVarRedn) {
+                if (useVarRedn) {
 
                     /* Variance reduction */
 
@@ -611,7 +621,7 @@ BasicCubefulRollout(unsigned int aanBoard[][2][25],
 
                         }
 
-                    if (prc->fInitial && !iTurn && j == i)
+                    if (prc->fInitial && !iTurn)
                         /* no doubles ... */
                         for (i = 0; i < NUM_ROLLOUT_OUTPUTS; i++)
                             arMean[i] /= 30.0f;
@@ -622,7 +632,7 @@ BasicCubefulRollout(unsigned int aanBoard[][2][25],
                     /* Find best move */
 
                     if (pecChequer[pci->fMove]->nPlies ||
-                        prc->fCubeful != pecChequer[pci->fMove]->fCubeful || pecChequer[pci->fMove]->rNoise)
+                        prc->fCubeful != pecChequer[pci->fMove]->fCubeful || pecChequer[pci->fMove]->rNoise > 0.0f)
 
                         /* the user requested n-ply (n>0). Another call to
                          * FindBestMove is required */
@@ -656,7 +666,7 @@ BasicCubefulRollout(unsigned int aanBoard[][2][25],
 
                         r = arMean[OUTPUT_CUBEFUL_EQUITY] - aaar[anDice[0] - 1][anDice[1] - 1]
                             [OUTPUT_CUBEFUL_EQUITY];
-                        aarVarRedn[ici][OUTPUT_CUBEFUL_EQUITY] += r * pci->nCube / aci[ici].nCube;
+                        aarVarRedn[ici][OUTPUT_CUBEFUL_EQUITY] += r * (float) (pci->nCube / aci[ici].nCube);
                     }
 
                 } else {
@@ -748,13 +758,13 @@ BasicCubefulRollout(unsigned int aanBoard[][2][25],
                     if (aarsStatistics)
                         switch (GameStatus((ConstTanBoard) aanBoard[ici], pci->bgv)) {
                         case 1:
-                            MT_SafeInc(&aarsStatistics[ici][pci->fMove].acWin[LogCube(pci->nCube)]);
+                            MT_SafeInc(&aarsStatistics[ici][pci->fMove].acWin[LogCubeClamped(pci->nCube)]);
                             break;
                         case 2:
-                            MT_SafeInc(&aarsStatistics[ici][pci->fMove].acWinGammon[LogCube(pci->nCube)]);
+                            MT_SafeInc(&aarsStatistics[ici][pci->fMove].acWinGammon[LogCubeClamped(pci->nCube)]);
                             break;
                         case 3:
-                            MT_SafeInc(&aarsStatistics[ici][pci->fMove].acWinBackgammon[LogCube(pci->nCube)]);
+                            MT_SafeInc(&aarsStatistics[ici][pci->fMove].acWinBackgammon[LogCubeClamped(pci->nCube)]);
                             break;
                         }
 
@@ -800,16 +810,16 @@ BasicCubefulRollout(unsigned int aanBoard[][2][25],
          * all variance reduction terms */
 
         if (!pci->nMatchTo)
-            aarOutput[ici][OUTPUT_CUBEFUL_EQUITY] *= pci->nCube / aci[ici].nCube;
+            aarOutput[ici][OUTPUT_CUBEFUL_EQUITY] *= (float) (pci->nCube / aci[ici].nCube);
 
-        if (prc->fVarRedn)
+        if (useVarRedn)
             for (i = 0; i < NUM_ROLLOUT_OUTPUTS; i++)
                 aarOutput[ici][i] += aarVarRedn[ici][i];
 
         /* multiply money equities */
 
         if (!pci->nMatchTo)
-            aarOutput[ici][OUTPUT_CUBEFUL_EQUITY] *= aci[ici].nCube / nBasisCube;
+            aarOutput[ici][OUTPUT_CUBEFUL_EQUITY] *= (float) (aci[ici].nCube / nBasisCube);
 
 
 
@@ -825,7 +835,7 @@ BasicCubefulRollout(unsigned int aanBoard[][2][25],
     return 0;
 }
 
-#if !LOCKING_VERSION
+#if !defined(LOCKING_VERSION)
 
 /* called with a collection of moves or a cube decision to be rolled out.
  * when called with a cube decision, the number of alternatives is always 2
@@ -902,7 +912,7 @@ comp_jsdinfo_order(const void *a, const void *b)
 }
 
 /* Lots of shared variables - should probably not be globals... */
-static unsigned int cGames;
+static int cGames;
 static cubeinfo *aciLocal;
 static int show_jsds;
 
@@ -930,13 +940,13 @@ check_jsds(int *active)
 {
     int alt;
     float v, s, denominator;
-    rolloutcontext *prc;
 
     for (alt = 0; alt < ro_alternatives; ++alt) {
 
         /* 1) For each move, calculate the cubeful (or cubeless if that's what we're doing)
          * equity */
-        prc = &ro_apes[alt]->rc;
+        rolloutcontext *prc = &ro_apes[alt]->rc;
+
         if (prc->fCubeful) {
             v = aarMu[alt][OUTPUT_CUBEFUL_EQUITY];
             s = aarSigma[alt][OUTPUT_CUBEFUL_EQUITY];
@@ -1057,31 +1067,31 @@ check_sds(int *active)
     int alt;
     for (alt = 0; alt < ro_alternatives; ++alt) {
         float s;
-        int output;
+        int ioutput;
         int err_too_big = 0;
         rolloutcontext *prc;
         if (fNoMore[alt] || altGameCount[alt] < (rcRollout.nMinimumGames))
             continue;
         prc = &ro_apes[alt]->rc;
-        for (output = OUTPUT_EQUITY; output < NUM_ROLLOUT_OUTPUTS; output++) {
-            if (output == OUTPUT_EQUITY) {      /* cubeless */
+        for (ioutput = OUTPUT_EQUITY; ioutput < NUM_ROLLOUT_OUTPUTS; ioutput++) {
+            if (ioutput == OUTPUT_EQUITY) {     /* cubeless */
                 if (!ms.nMatchTo) {     /* money game */
-                    s = fabsf(aarSigma[alt][output]);
+                    s = fabsf(aarSigma[alt][ioutput]);
                     if (ro_fCubeRollout) {
-                        s *= aciLocal[alt].nCube / aciLocal[0].nCube;
+                        s *= (float) (aciLocal[alt].nCube / aciLocal[0].nCube);
                     }
                 } else {        /* match play */
-                    s = fabsf(se_mwc2eq(se_eq2mwc(aarSigma[alt][output],
-                                                 &aciLocal[alt]), &aciLocal[(ro_fCubeRollout ? 0 : alt)]));
+                    s = fabsf(se_mwc2eq(se_eq2mwc(aarSigma[alt][ioutput],
+                                                  &aciLocal[alt]), &aciLocal[(ro_fCubeRollout ? 0 : alt)]));
                 }
             } else {
                 if (!prc->fCubeful)
                     continue;
                 /* cubeful */
                 if (!ms.nMatchTo) {     /* money game */
-                    s = fabsf(aarSigma[alt][output]);
+                    s = fabsf(aarSigma[alt][ioutput]);
                 } else {
-                    s = fabsf(se_mwc2eq(aarSigma[alt][output], &aciLocal[(ro_fCubeRollout ? 0 : alt)]));
+                    s = fabsf(se_mwc2eq(aarSigma[alt][ioutput], &aciLocal[(ro_fCubeRollout ? 0 : alt)]));
                 }
             }
 
@@ -1089,7 +1099,7 @@ check_sds(int *active)
                 err_too_big = 1;
                 break;
             }
-        }                       /* for (output = OUTPUT_EQUITY; output < NUM_ROLLOUT_OUTPUTS; output++) */
+        }                       /* for (ioutput = OUTPUT_EQUITY; ioutput < NUM_ROLLOUT_OUTPUTS; ioutput++) */
 
         if (!err_too_big) {
             fNoMore[alt] = 1;
@@ -1114,7 +1124,6 @@ RolloutLoopMT(void *UNUSED(unused))
     int active_alternatives;
     unsigned int j;
     int alt;
-    unsigned int this_trial;
     FILE *logfp = NULL;
     rolloutcontext *prc = NULL;
     /* Each thread gets a copy of the rngctxRollout */
@@ -1124,11 +1133,11 @@ RolloutLoopMT(void *UNUSED(unused))
 
     /* ============ begin rollout loop ============= */
 
-    while ((this_trial = MT_SafeIncValue(&ro_NextTrial)) <= cGames) {
+    while (MT_SafeIncValue(&ro_NextTrial) <= cGames) {
         active_alternatives = ro_alternatives;
 
         for (alt = 0; alt < ro_alternatives; ++alt) {
-            unsigned int trial = MT_SafeIncValue(&altTrialCount[alt]) - 1;
+            int trial = MT_SafeIncValue(&altTrialCount[alt]) - 1;
             /* skip this one if it's already finished */
             if (fNoMore[alt] || (trial > cGames)) {
                 MT_SafeDec(&altTrialCount[alt]);
@@ -1142,7 +1151,7 @@ RolloutLoopMT(void *UNUSED(unused))
             if (prc->fRotate)
                 QuasiRandomSeed(&dicePerms, (int) prc->nSeed);
 
-            nSkip = 0;          /* not multi-thread safe do quasi random dice for initial positions */
+            MT_SafeSet(&nSkip, 0);      /* not multi-thread safe do quasi random dice for initial positions */
 
             /* ... and the RNG */
             if (prc->rngRollout != RNG_MANUAL)
@@ -1177,18 +1186,17 @@ RolloutLoopMT(void *UNUSED(unused))
 
             /* apply the results */
             for (j = 0; j < NUM_ROLLOUT_OUTPUTS; j++) {
-                float rMuNew, rDelta;
+                float rMuNew;
 
                 aarResult[alt][j] += aar[j];
-                rMuNew = aarResult[alt][j] / (altGameCount[alt]);
+                rMuNew = aarResult[alt][j] / (float) altGameCount[alt];
 
                 if (altGameCount[alt] > 1) {    /* for i == 0 aarVariance is not defined */
-
-                    rDelta = rMuNew - aarMu[alt][j];
+                    float rDelta = rMuNew - aarMu[alt][j];
 
                     aarVariance[alt][j] =
-                        aarVariance[alt][j] * (1.0f - 1.0f / (altGameCount[alt] - 1)) +
-                        (altGameCount[alt]) * rDelta * rDelta;
+                        aarVariance[alt][j] * (1.0f - 1.0f / (float) (altGameCount[alt] - 1)) +
+                        (float) (altGameCount[alt]) * rDelta * rDelta;
                 }
 
                 aarMu[alt][j] = rMuNew;
@@ -1200,7 +1208,7 @@ RolloutLoopMT(void *UNUSED(unused))
                         aarMu[alt][j] = 1.0f;
                 }
 
-                aarSigma[alt][j] = sqrtf(aarVariance[alt][j] / (altGameCount[alt]));
+                aarSigma[alt][j] = sqrtf(aarVariance[alt][j] / (float) altGameCount[alt]);
             }                   /* for (j = 0; j < NUM_ROLLOUT_OUTPUTS; j++ ) */
 
             /* For normal alternatives nGamesDone and altGameCount will be equal. For cube decisions,
@@ -1221,7 +1229,7 @@ RolloutLoopMT(void *UNUSED(unused))
         /* Stop rolling out moves whose Equity is more than a user selected multiple of the joint standard
          * deviation of the equity difference with the best move in the list. */
 
-#if !USE_MULTITHREAD
+#if !defined(USE_MULTITHREAD)
         ProcessEvents();
 #endif
 
@@ -1241,7 +1249,7 @@ RolloutLoopMT(void *UNUSED(unused))
         multi_debug("exclusive release: rollout cycle update");
         MT_Release();
     }
-    free(rngctxMTRollout);
+    g_free(rngctxMTRollout);
 }
 
 static rolloutprogressfunc *ro_pfProgress;
@@ -1252,13 +1260,13 @@ UpdateProgress(gpointer UNUSED(unused))
 {
     if (fShowProgress && ro_alternatives > 0) {
         int alt;
-        rolloutcontext *prc;
 
         multi_debug("exclusive lock: update progress");
         MT_Exclusive();
 
         for (alt = 0; alt < ro_alternatives; ++alt) {
-            prc = &ro_apes[alt]->rc;
+            rolloutcontext *prc = &ro_apes[alt]->rc;
+
             (*ro_pfProgress) (aarMu, aarSigma, prc, aciLocal, initial_game_count, altGameCount[alt] - 1, alt,
                               ajiJSD[alt].nRank + 1, ajiJSD[alt].rJSD, fNoMore[alt], show_jsds, ro_fCubeRollout,
                               ro_pUserData);
@@ -1380,7 +1388,6 @@ RolloutGeneral(ConstTanBoard * apBoard,
             }
         } else {
             int nGames = prc->nGamesDone;
-            float r;
 
             previous_rollouts++;
 
@@ -1397,10 +1404,12 @@ RolloutGeneral(ConstTanBoard * apBoard,
                 nFirstTrial = nGames;
             /* restore internal variables from input values */
             for (j = 0; j < NUM_ROLLOUT_OUTPUTS; ++j) {
+                float r;
+
                 r = aarMu[alt][j] = (*apOutput[alt])[j];
-                aarResult[alt][j] = r * nGames;
+                aarResult[alt][j] = r * (float) nGames;
                 r = aarSigma[alt][j] = (*apStdDev[alt])[j];
-                aarVariance[alt][j] = r * r * nGames;
+                aarVariance[alt][j] = r * r * (float) nGames;
             }
         }
 
@@ -1467,8 +1476,8 @@ RolloutGeneral(ConstTanBoard * apBoard,
         multi_debug("rollout finished waiting for tasks to complete");
     }
 
-    /* Make sure final output is upto date */
-#if USE_GTK
+    /* Make sure final output is up to date */
+#if defined(USE_GTK)
     if (!fX)
 #endif
         if (!fInterrupt)
@@ -1506,7 +1515,7 @@ RolloutGeneral(ConstTanBoard * apBoard,
     }
 
     if (fShowProgress && !fInterrupt
-#if USE_GTK
+#if defined(USE_GTK)
         && !fX
 #endif
         ) {
@@ -1569,7 +1578,7 @@ GeneralEvaluationR(float arOutput[NUM_ROLLOUT_OUTPUTS],
     evalsetup es;
     evalsetup(*apes[1]);
     const cubeinfo(*apci[1]);
-    int false = 0;
+    int cubedectop = 0;
     int (*apCubeDecTop[1]);
 
     apBoard[0] = anBoard;
@@ -1577,7 +1586,7 @@ GeneralEvaluationR(float arOutput[NUM_ROLLOUT_OUTPUTS],
     apStdDev[0] = (float (*)[NUM_ROLLOUT_OUTPUTS]) arStdDev;
     apes[0] = &es;
     apci[0] = pci;
-    apCubeDecTop[0] = &false;
+    apCubeDecTop[0] = &cubedectop;
 
     es.et = EVAL_NONE;
     memcpy(&es.rc, prc, sizeof(rolloutcontext));
@@ -1632,13 +1641,11 @@ GeneralCubeDecisionR(float aarOutput[2][NUM_ROLLOUT_OUTPUTS],
                      const TanBoard anBoard,
                      cubeinfo * pci, rolloutcontext * prc, evalsetup * pes, rolloutprogressfunc * pf, void *p)
 {
-
-
     evalsetup esLocal;
     evalsetup(*apes[2]);
     cubeinfo aci[2];
     const cubeinfo(*apci[2]);
-    int cGames;
+    int nTrials;
     int afCubeDecTop[] = { FALSE, FALSE };      /* no cube decision in 
                                                  * iTurn = 0 */
     ConstTanBoard apBoard[2];
@@ -1681,16 +1688,14 @@ GeneralCubeDecisionR(float aarOutput[2][NUM_ROLLOUT_OUTPUTS],
         prc->fCubeful = TRUE;
     }
 
-
-    if ((cGames = RolloutGeneral(apBoard, apOutput, apStdDev,
-                                 aarsStatistics, apes, apci, apCubeDecTop, 2, FALSE, TRUE, pf, p)) <= 0)
+    if ((nTrials = RolloutGeneral(apBoard, apOutput, apStdDev,
+                                  aarsStatistics, apes, apci, apCubeDecTop, 2, FALSE, TRUE, pf, p)) <= 0)
         return -1;
 
-    pes->rc.nGamesDone = cGames;
-    pes->rc.nSkip = nSkip;
+    pes->rc.nGamesDone = nTrials;
+    pes->rc.nSkip = MT_SafeGet(&nSkip);
 
     return 0;
-
 }
 
 
@@ -1709,7 +1714,7 @@ GeneralCubeDecisionR(float aarOutput[2][NUM_ROLLOUT_OUTPUTS],
  *
  */
 
-extern void
+static void
 initRolloutstat(rolloutstat * prs)
 {
 
@@ -1803,14 +1808,11 @@ getResignEquities(float arResign[NUM_ROLLOUT_OUTPUTS], cubeinfo * pci, int nResi
 
 
 extern int
-ScoreMoveRollout(move ** ppm, const cubeinfo ** ppci, int cMoves, rolloutprogressfunc * pf, void *p)
+ScoreMoveRollout(move ** ppm, cubeinfo ** ppci, int cMoves, rolloutprogressfunc * pfRolloutProgress, void * pUserData)
 {
-
-    const cubeinfo *pci;
     int fCubeDecTop = TRUE;
     int i;
     int nGamesDone;
-    rolloutcontext *prc;
 
     TanBoard *anBoard = g_alloca(cMoves * 2 * 25 * sizeof(int));
     ConstTanBoard *apBoard = g_alloca(cMoves * sizeof(int *));
@@ -1837,11 +1839,11 @@ ScoreMoveRollout(move ** ppm, const cubeinfo ** ppci, int cMoves, rolloutprogres
 
         /* swap fMove in cubeinfo */
         aci[i].fMove = !aci[i].fMove;
-
     }
 
     nGamesDone = RolloutGeneral(apBoard,
-                                apOutput, apStdDev, NULL, apes, apci, apCubeDecTop, cMoves, TRUE, FALSE, pf, p);
+                                apOutput, apStdDev, NULL, apes, apci, apCubeDecTop, cMoves, TRUE, FALSE,
+                                pfRolloutProgress, pUserData);
     /* put fMove back again */
     for (i = 0; i < cMoves; ++i) {
         aci[i].fMove = !aci[i].fMove;
@@ -1851,6 +1853,8 @@ ScoreMoveRollout(move ** ppm, const cubeinfo ** ppci, int cMoves, rolloutprogres
         return -1;
 
     for (i = 0; i < cMoves; ++i) {
+        const cubeinfo *pci;
+        rolloutcontext *prc;
 
         /* Score for move:
          * rScore is the primary score (cubeful/cubeless)
@@ -1872,4 +1876,18 @@ ScoreMoveRollout(move ** ppm, const cubeinfo ** ppci, int cMoves, rolloutprogres
     return 0;
 }
 
+extern void
+InvertStdDev(float ar[NUM_ROLLOUT_OUTPUTS])
+{
+    float r;
+
+    r = ar[OUTPUT_WINGAMMON];
+    ar[OUTPUT_WINGAMMON] = ar[OUTPUT_LOSEGAMMON];
+    ar[OUTPUT_LOSEGAMMON] = r;
+
+    r = ar[OUTPUT_WINBACKGAMMON];
+    ar[OUTPUT_WINBACKGAMMON] = ar[OUTPUT_LOSEBACKGAMMON];
+    ar[OUTPUT_LOSEBACKGAMMON] = r;
+}
+
 #endif
