@@ -15,7 +15,7 @@
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
- * $Id: show.c,v 1.311 2023/12/20 23:09:55 plm Exp $
+ * $Id: show.c,v 1.313 2024/02/18 16:30:31 plm Exp $
  */
 
 #include "config.h"
@@ -732,7 +732,7 @@ CommandShowEvaluation(char *UNUSED(sz))
 extern void
 CommandShowHistory(char *UNUSED(sz))
 {
-    ComputeHistory();
+    ComputeHistory(FALSE);
 }
 #endif
 
@@ -1296,7 +1296,7 @@ show_isight(TanBoard an, char *sz)
 
     gwc = 80.0f - (float) pn[1] / 3.0f + 2.0f * (float) (pn[0] - pn[1]);
 
-    sprintf(strchr(sz, 0), _("Estimated GWC: %4.1f%% (80 - L/3 + 2*(L-T))\n\n"), gwc);
+    sprintf(strchr(sz, 0), _("Estimated GWC: %4.1f%% (80 - L/3 + 2*(T-L))\n\n"), gwc);
 
     if (gwc > 76.0f)
         sprintf(strchr(sz, 0), _("Double, Drop (since est. GWC > 76%%)\n"));
